/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
// Written by Mark Hamzy

#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <time.h>

#include <assrt.h>

typedef struct _MyTime {
   INT twosecs;
   INT minutes;
   INT hours;
} MYTIME, *PMYTIME;

typedef struct _MyDate {
   INT day;
   INT month;
   INT year;
} MYDATE, *PMYDATE;

/* Function prototypes...
*/
PSZ           pszMonth                   (INT      iMonth);
PSZ           pszDay                     (INT      iDay);
VOID          PrintUsage                 (PSZ      pszProgramName);
VOID          CopyFromDate               (FDATE    fdateIn,
                                          PMYDATE  pmyDateOut);
VOID          CopyFromTime               (FTIME    ftimeIn,
                                          PMYTIME  pmyTimeOut);
VOID          CopyToDate                 (MYDATE   myDateIn,
                                          PFDATE   pfdateOut);
VOID          CopyToTime                 (MYTIME   myTimeIn,
                                          PFTIME   pftimeOut);
VOID          AddDateTime                (PMYTIME  pmyTimeTo,
                                          PMYDATE  pmyDateTo,
                                          MYTIME   myTimeFrom,
                                          MYDATE   myDateFrom);
VOID          SplitDirectoryNameFromPath (PSZ      pszFullyQualifiedPath);
BOOL          HandleDirectory            (PSZ      pszDirectory,
                                          PMYDATE  pmyDateBump,
                                          PMYTIME  pmyTimeBump);
BOOL          HandleFiles                (PSZ      pszDirectory,
                                          PMYDATE  pmyDateBump,
                                          PMYTIME  pmyTimeBump);
BOOL          HandleFile                 (PSZ      pszDirectory,
                                          PMYDATE  pmyDateBump,
                                          PMYTIME  pmyTimeBump);

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
PSZ
pszMonth (INT iMonth)
{
   static PSZ aMonths[] = {
      "Jan", "Feb", "Mar", "Apr", "May", "Jun",
      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
   };

   assertF (0 <= iMonth && iMonth < sizeof (aMonths));
   return aMonths[iMonth];
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
PSZ
pszDay (INT iDay)
{
   static PSZ aDays[] = {
      "Sunday",
      "Monday",
      "Tuesday",
      "Wednesday",
      "Thursday",
      "Friday",
      "Saturday"
   };

   assertF (0 <= iDay && iDay < sizeof (aDays));
   return aDays[iDay];
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
CopyFromDate (FDATE fdateIn, PMYDATE pmyDateOut)
{
   pmyDateOut->day   = fdateIn.day;
   pmyDateOut->month = fdateIn.month;
   pmyDateOut->year  = fdateIn.year;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
CopyFromTime (FTIME ftimeIn, PMYTIME pmyTimeOut)
{
   pmyTimeOut->twosecs = ftimeIn.twosecs;
   pmyTimeOut->minutes = ftimeIn.minutes;
   pmyTimeOut->hours   = ftimeIn.hours;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
CopyToDate (MYDATE myDateIn, PFDATE pfdateOut)
{
   pfdateOut->day   = myDateIn.day;
   pfdateOut->month = myDateIn.month;
   pfdateOut->year  = myDateIn.year;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
CopyToTime (MYTIME myTimeIn, PFTIME pftimeOut)
{
   pftimeOut->twosecs = myTimeIn.twosecs;
   pftimeOut->minutes = myTimeIn.minutes;
   pftimeOut->hours   = myTimeIn.hours;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
SetDaysInLeapYear (int *piDays, int iYear)
{
   if (iYear % 4 != 0 ? 0 : iYear % 100 != 0 ? 1 : iYear % 400 != 0 ? 0 : 1)
      *piDays = 29;
   else
      *piDays = 28;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID
AddDateTime (PMYTIME pmyTimeTo,
             PMYDATE pmyDateTo,
             MYTIME  myTimeFrom,
             MYDATE  myDateFrom)
{
   long ltwosecs;
   long ltwosecsinc;
   long lminutes;
   long lminutesinc;
   long lhours;
   long lhoursinc;
   long lday;
   long ldayinc;
   long lmonth;
   long lmonthinc;
   long lyear;         //  J   F   M   A   M   J   J   A   S   O   N   D
   int  aMaxDays[13] = {0, 31, 28, 31, 30, 30, 31, 31, 31, 30, 31, 30, 31 };

   ltwosecs    = pmyTimeTo->twosecs;
   ltwosecsinc = myTimeFrom.twosecs;
   lminutesinc = myTimeFrom.minutes;
   while (0 != ltwosecsinc)
   {
      if (0 > ltwosecsinc)
      {
         if (-ltwosecsinc > ltwosecs)
         {
            ltwosecsinc += ltwosecs;
            lminutesinc--;
            ltwosecs = 30;
         }
         else
         {
            ltwosecs += ltwosecsinc;
            ltwosecsinc = 0;                               // Done!
         }
      }
      else
      {
         if (ltwosecs + ltwosecsinc > 29)
         {
            ltwosecsinc -= 30 - ltwosecs;
            lminutesinc++;
            ltwosecs = 0;
         }
         else
         {
            ltwosecs += ltwosecsinc;
            ltwosecsinc = 0;                               // Done!
         }
      }
   }
   pmyTimeTo->twosecs = ltwosecs;
   myTimeFrom.minutes = lminutesinc;

   lminutes    = pmyTimeTo->minutes;
   lminutesinc = myTimeFrom.minutes;
   lhoursinc   = myTimeFrom.hours;
   while (0 != lminutesinc)
   {
      if (0 > lminutesinc)
      {
         if (-lminutesinc > lminutes)
         {
            lminutesinc += lminutes;
            lhoursinc--;
            lminutes = 60;
         }
         else
         {
            lminutes += lminutesinc;
            lminutesinc = 0;                               // Done!
         }
      }
      else
      {
         if (lminutes + lminutesinc > 59)
         {
            lminutesinc -= 60 - lminutes;
            lhoursinc++;
            lminutes = 0;
         }
         else
         {
            lminutes += lminutesinc;
            lminutesinc = 0;                               // Done!
         }
      }
   }
   pmyTimeTo->minutes = lminutes;
   myTimeFrom.hours   = lhoursinc;

   lhours    = pmyTimeTo->hours;
   lhoursinc = myTimeFrom.hours;
   ldayinc   = myDateFrom.day;
   while (0 != lhoursinc)
   {
      if (0 > lhoursinc)
      {
         if (-lhoursinc > lhours)
         {
            lhoursinc += lhours;
            ldayinc--;
            lhours = 24;
         }
         else
         {
            lhours += lhoursinc;
            lhoursinc = 0;                                 // Done!
         }
      }
      else
      {
         if (lhours + lhoursinc > 23)
         {
            lhoursinc -= 24 - lhours;
            ldayinc++;
            lhours = 0;
         }
         else
         {
            lhours += lhoursinc;
            lhoursinc = 0;                                 // Done!
         }
      }
   }
   pmyTimeTo->hours = lhours;
   myDateFrom.day   = ldayinc;

   lday    = pmyDateTo->day;
   ldayinc = myDateFrom.day;
   lmonth  = pmyDateTo->month;
   lyear   = pmyDateTo->year + 1980;
   SetDaysInLeapYear (&aMaxDays[2], lyear);
   while (0 != ldayinc)
   {
      if (0 > ldayinc)
      {
         if (-ldayinc >= lday)
         {
            ldayinc += lday;

            lmonth--;
            if (0 == lmonth)
            {
               lmonth = 12;
               lyear--;
               SetDaysInLeapYear (&aMaxDays[2], lyear);
            }
            lday = aMaxDays[lmonth];
         }
         else
         {
            lday += ldayinc;
            ldayinc = 0;                                   // Done!
         }
      }
      else
      {
         if (lday + ldayinc > aMaxDays[lmonth])
         {
            ldayinc -= aMaxDays[lmonth] - lday;

            lmonth++;
            if (13 == lmonth)
            {
               lmonth = 1;
               lyear++;
               SetDaysInLeapYear (&aMaxDays[2], lyear);
            }
            lday = 1;
         }
         else
         {
            lday += ldayinc;
            ldayinc = 0;                                   // Done!
         }
      }
   }
   pmyDateTo->day   = lday;
   pmyDateTo->month = lmonth;
   pmyDateTo->year  = lyear - 1980;

   lmonth    = pmyDateTo->month;
   lmonthinc = myDateFrom.month;
   lyear     = pmyDateTo->year + 1980;
   while (0 != lmonthinc)
   {
      if (0 > lmonthinc)
      {
         if (-lmonthinc >= lmonth)
         {
            lmonthinc += lmonth;
            lmonth = 12;
            lyear--;
         }
         else
         {
            lmonth += lmonthinc;
            lmonthinc = 0;                                 // Done!
         }
      }
      else
      {
         if (lmonth + lmonthinc > 12)
         {
            lmonthinc -= 12 - lmonth;
            lmonth = 1;
            lyear++;
         }
         else
         {
            lmonth += lmonthinc;
            lmonthinc = 0;                                 // Done!
         }
      }
   }
   pmyDateTo->month = lmonth;
   pmyDateTo->year  = lyear - 1980;

   lyear = pmyDateTo->year + myDateFrom.year;
   if (0 > lyear)
      lyear = 0;
   pmyDateTo->year = lyear;
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
VOID
SplitDirectoryNameFromPath (PSZ pszFullyQualifiedPath)
{
   PSZ   pszFQP = pszFullyQualifiedPath;
   INT   iStrlen;

   assertF (pszFullyQualifiedPath);
   assertF (*pszFullyQualifiedPath);

   if (!pszFullyQualifiedPath || !*pszFullyQualifiedPath)
      return;

   // Find the rightmost slash
   for (iStrlen = strlen (pszFQP) - 1;
        0 <= iStrlen && '\\' != pszFQP[iStrlen];
        )
      iStrlen--;

   // did we find no slashes?
   if (0 > iStrlen)
   {
      // No directory present!
      strcpy (pszFQP, ".");

      return;
   }

   // remove the slash if it is not infront of a colon
   if (1 < iStrlen && ':' != pszFQP[iStrlen - 1])
   {
      pszFQP[iStrlen] = 0;
   }
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
BOOL
HandleDirectory (char    *pszDirectoryWild,
                 PMYDATE  pmyDateBump,
                 PMYTIME  pmyTimeBump)
{
   CHAR          achDirectory[CCHMAXPATH];
   CHAR          achFile[CCHMAXPATH];
   FILEFINDBUF4  FileBuffer;
   HDIR          FileHandle;
   ULONG         ulFileCount;
   APIRET        rc;
   BOOL          fRetVal                 = FALSE,   // FALSE is success!
                 fRet;

   strcpy (achFile, pszDirectoryWild);
   strcpy (achDirectory, pszDirectoryWild);
   SplitDirectoryNameFromPath (achDirectory);

   // Loop through the directories
   ulFileCount = 1;
   FileHandle = HDIR_CREATE;
   rc = DosFindFirst (achFile,
                      &FileHandle,
                      MUST_HAVE_DIRECTORY
                      | FILE_ARCHIVED
                      | FILE_DIRECTORY
                      | FILE_SYSTEM
                      | FILE_HIDDEN
                      | FILE_READONLY,
                      (VOID *)&FileBuffer,
                      sizeof (FileBuffer),
                      &ulFileCount,
                      FIL_QUERYEASIZE);

   while (NO_ERROR == rc)
   {
      // Dont do . and ..
      if (0 != strcmp (FileBuffer.achName, ".") &&
          0 != strcmp (FileBuffer.achName, "..") )
      {
         sprintf (achFile, "%s\\%s\\*", achDirectory, FileBuffer.achName);

         // Recurse!
         fRet = HandleDirectory (achFile, pmyDateBump, pmyTimeBump);
         fRetVal = fRetVal || fRet;
      }

      // Get next directory
      ulFileCount = 1;
      rc = DosFindNext (FileHandle,
                        (VOID *)&FileBuffer,
                        sizeof (FileBuffer),
                        &ulFileCount);
   }

   // Now it is safe to handle all of the files in this directory
   fRet = HandleFiles (pszDirectoryWild, pmyDateBump, pmyTimeBump);
   fRetVal = fRetVal || fRet;

   // Clean up
   rc = DosFindClose (FileHandle);

   return fRetVal;
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
BOOL
HandleFiles (char    *pszDirectoryWild,
             PMYDATE  pmyDateBump,
             PMYTIME  pmyTimeBump)
{
   CHAR          achDirectory[CCHMAXPATH];
   CHAR          achFile[CCHMAXPATH];
   FILEFINDBUF4  FileBuffer;
   HDIR          FileHandle;
   ULONG         ulFileCount;
   APIRET        rc;
   BOOL          fRetVal                 = FALSE,   // FALSE is success!
                 fRet;

   strcpy (achDirectory, pszDirectoryWild);
   SplitDirectoryNameFromPath (achDirectory);

   // Loop through the files
   ulFileCount = 1;
   FileHandle  = HDIR_CREATE;
   rc = DosFindFirst (pszDirectoryWild,
                      &FileHandle,
                      FILE_ARCHIVED
                      | FILE_SYSTEM
                      | FILE_HIDDEN
                      | FILE_READONLY,
                      (VOID *)&FileBuffer,
                      sizeof (FileBuffer),
                      &ulFileCount,
                      FIL_QUERYEASIZE);

   while (NO_ERROR == rc)
   {
      sprintf (achFile, "%s\\%s", achDirectory, FileBuffer.achName);

      fRet = HandleFile (achFile, pmyDateBump, pmyTimeBump);
      fRetVal = fRetVal || fRet;

      // Get next file
      ulFileCount = 1;
      rc = DosFindNext (FileHandle,
                        (VOID *)&FileBuffer,
                        sizeof (FileBuffer),
                        &ulFileCount);
   }

   // Clean up
   rc = DosFindClose (FileHandle);

   return fRetVal;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
BOOL
HandleFile (PSZ pszFileName, PMYDATE pmyDateBump, PMYTIME pmyTimeBump)
{
   FILESTATUS3   FileStatus;
   MYDATE        myDateOrig;
   MYTIME        myTimeOrig;
   struct tm     tm;
   APIRET        rc;

   // Query info
   rc = DosQueryPathInfo (pszFileName,
                          FIL_STANDARD,
                          (VOID *)&FileStatus,
                          sizeof (FileStatus));
   if (NO_ERROR != rc)
   {
      fprintf (stderr, "DosQueryPathInfo (%s) rc = %d ", pszFileName, rc);
   }

   CopyFromDate (FileStatus.fdateLastWrite, &myDateOrig);
   CopyFromTime (FileStatus.ftimeLastWrite, &myTimeOrig);

   AddDateTime (&myTimeOrig,
                &myDateOrig,
                *pmyTimeBump,
                *pmyDateBump);

   CopyToDate (myDateOrig, &FileStatus.fdateLastWrite);
   CopyToTime (myTimeOrig, &FileStatus.ftimeLastWrite);

   if (NO_ERROR == rc)
   {
      rc = DosSetPathInfo (pszFileName,
                           FIL_STANDARD,
                           (VOID *)&FileStatus,
                           sizeof (FileStatus),
                           DSPI_WRTTHRU);

      if (NO_ERROR != rc)
      {
         fprintf (stderr, "DosSetPathInfo rc = %d ", rc);

         return TRUE;
      }

      // Query info
      rc = DosQueryPathInfo (pszFileName,
                             FIL_STANDARD,
                             (VOID *)&FileStatus,
                             sizeof (FileStatus));
      if (NO_ERROR != rc)
      {
         fprintf (stderr, "Dos(re)QueryPathInfo (%s) rc = %d ", pszFileName, rc);
      }

      tm.tm_sec  = FileStatus.ftimeLastWrite.twosecs * 2;
      tm.tm_min  = FileStatus.ftimeLastWrite.minutes;
      tm.tm_hour = FileStatus.ftimeLastWrite.hours;
      tm.tm_mday = FileStatus.fdateLastWrite.day;
      tm.tm_mon  = FileStatus.fdateLastWrite.month - 1;
      tm.tm_year = 80 + FileStatus.fdateLastWrite.year;

      mktime (&tm);

      fprintf (stdout, "%s bumped to %s %s %d %02d:%02d:%02d %04d\n",
               pszFileName,
               pszDay (tm.tm_wday),
               pszMonth (tm.tm_mon),
               FileStatus.fdateLastWrite.day,
               FileStatus.ftimeLastWrite.hours,
               FileStatus.ftimeLastWrite.minutes,
               FileStatus.ftimeLastWrite.twosecs * 2,
               1980 + FileStatus.fdateLastWrite.year);

      return FALSE;
   }

   return TRUE;
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
int
main (int argc, char *argv[], char *envp[])
{
   PSZ           pszFileName          = (PSZ)NULL;
   MYDATE        myDateBump;
   MYTIME        myTimeBump;
   int           iLastNumber;
   BOOL          fRetVal       = TRUE;
   register int  i;

   HookAssert (HOOKTYPE_PRINTF);

   memset (&myDateBump, 0, sizeof (myDateBump));
   memset (&myTimeBump, 0, sizeof (myTimeBump));

   // Read the parameters
   for (i = 1; i < argc; i++)
   {
      DBPRINTF (("Examining argument #%d '%s'\n", i, argv[i]));

      if (atoi (argv[i]))
      {
         iLastNumber = atoi (argv[i]);
         DBPRINTF (("iLastNumber = %d\n", iLastNumber));
      }
      else if (0 == stricmp (argv[i], "seconds") ||
               0 == stricmp (argv[i], "second")   )
         myTimeBump.twosecs += iLastNumber / 2;
      else if (0 == stricmp (argv[i], "minutes") ||
               0 == stricmp (argv[i], "minute")   )
         myTimeBump.minutes += iLastNumber;
      else if (0 == stricmp (argv[i], "hours") ||
               0 == stricmp (argv[i], "hour")   )
         myTimeBump.hours += iLastNumber;
      else if (0 == stricmp (argv[i], "days") ||
               0 == stricmp (argv[i], "day")   )
         myDateBump.day += iLastNumber;
      else if (0 == stricmp (argv[i], "months") ||
               0 == stricmp (argv[i], "month")   )
         myDateBump.month += iLastNumber;
      else if (0 == stricmp (argv[i], "years") ||
               0 == stricmp (argv[i], "year")   )
         myDateBump.year += iLastNumber;
      else if (!pszFileName)
         pszFileName = argv[i];
      else
      {
         PrintUsage (argv[0]);

         goto done;
      }
   }

   if (!pszFileName)
   {
      PrintUsage (argv[0]);

      goto done;
   }

   fRetVal = HandleDirectory (pszFileName, &myDateBump, &myTimeBump);

done:
   return fRetVal;
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/
VOID
PrintUsage (char *pszProgramName)
{
   fprintf (stderr, "Usage: %s filename time_increment\n", pszProgramName);
   fprintf (stderr, "       where time_increment can be one or more:\n");
   fprintf (stderr, "          [-]number { seconds | minutes | hours |\n");
   fprintf (stderr, "                      days | months | years }\n");
   fprintf (stderr, "Example: %s bob 1 minutes -1 days\n");
}
