/* REXX:
** RXEA.CMD
**
** TBD -  rxea filename
**          should enumerate all eas for filename
**
** 1A
** 00 00 00 00      ULONG   oNextEntryOffset;
** 04               BYTE    cbName;
** 0D 00
** 42 49 4C 4C 00   CHAR    szName[1];
** DF FF            EAT_MVMT  0xFFDF /* multi-valued, multi-typed field */
** 00 00 01 00
** FD FF            EAT_ASCII 0xFFFD /* length preceeded ASCII */
** 03 00            USHORT  cbValue;
** 62 6F 62
**
** cbName  = 8
** cbValue = 16
** Name = .VERSION
** Value =
** ff ff 00 00 01 00 ff ff 06 00 33 30 2e 34 34 34      [..........30.444]
**
** cbName  = 10
** cbValue = 14
** Name = VENDORNAME
** Value =
** ff ff 00 00 01 00 ff ff 04 00 4f 4d 4e 49            [..........OMNI  ]
**
** cbName  = 19
** cbValue = 18
** Name = REQUIREDDRIVERFILES
** Value =
** ff ff 00 00 01 00 ff ff 08 00 4f 4d 4e 49 2e 44      [..........OMNI.D]
** 52 56                                                [RV              ]
*/

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

/* Parse program arguments
*/
parse arg szCommand szRest
szCommand = translate( szCommand )

/* Filename is the last parameter
*/
iNumWords  = words( szRest )
if iNumWords > 0 then
do
   szFileName = word( szRest, iNumWords )
   szRest     = substr( szRest, 1, wordindex( szRest, iNumWords ) - 1 )
end
else
   szFileName = szCommand /* possible... */

bRecognized = 0

/* is it the help option?
*/
if szCommand == "-?" then
   signal usage

/* File name is mandatory
*/
if \fValidFile( szFileName ) then
   signal usage

/* Is it the edit new/existing EA value ?
*/
if szCommand == "-E" then
do
   bRecognized = 1

   parse var szRest szEAName '=' szEAValue
   szEAName  = fMassage( szEAName )
   szEAValue = fMassage( szEAValue )

   /* Get the old value */
   rc = SysGetEA( szFileName, szEAName, "bInfo" )

   /* Set up the block of binary data
   ** This does require some bit twiddling...
   */
   iHigh  = length( szEAValue ) % 256
   iLow   = length( szEAValue ) - iHigh * 256
   bStuff = "DFFF"x      ||,   /* EAT_MVMT  0xFFDF multi-valued, multi-typed field */
            "00000100"x  ||,   /* ??? */
            "FDFF"x      ||,   /* EAT_ASCII 0xFFFD length preceeded ASCII          */
            d2c( iLow )  ||,
            d2c( iHigh )

   /* Set the new value */
   rc = SysPutEA( szFileName, szEAName, bStuff || szEAValue )
end

/* Is it the delete existing EA option?
*/
if szCommand == "-D" then
do
   bRecognized = 1

   parse var szRest szEAName
   szEAName = fMassage( szEAName )

   /* Delete the EA */
   rc = SysPutEA( szFileName, szEAName, "" )
end

/* Is it not a -E or a -D?
*/
if \bRecognized & substr( szCommand, 1, 1 ) = "-" then
   signal usage

/* @TBD
** If no options and a filename then we need to enumerate all EAs for
** filename
*/

done:
call SysDropFuncs

exit

/***************************************************************************
**
** Print help
*/
usage:
parse source . . szName .
szName = filespec( "name", szName )

say "Usage:" szName "[-e name=value | -d name | ] file"
say "        -e = Edit new/existing EA value."
say "        -d = Delete exiting EA."
say "        -? = Show this help information."
say " Otherwise = Display all EAs."
say ""
say " To include a blank in the name, value, or file parameter,"
say " surround the string with single or double quote marks."
say ""
say " Examples:"
say "   "szName" file.dat"
say "   "szName" -e 'my pie'='apple and cherry' file.dat"
say "   "szName" -e language="french german" file.dat"
say "   "szName" -e language=french file.dat"
say "   "szName" -d language file.dat"

signal done

/***************************************************************************
**
** Is the file name valid (does it exist?)
*/
fValidFile: procedure
  arg szFileName

if szFileName = "" then
   return 0

rc = SysFileTree( szFileName, 'aStem', 'FT' )
if rc \= 0 then
   return 0
else
do
   if aStem.0 = 1 then
      return 1
   else
      return 0
end

/***************************************************************************
**
** Massage parameters
*/
fMassage: procedure
  parse arg szParameter

szParameter = strip( szParameter )

if substr( szParameter, 1, 1 ) == '"' | ,
   substr( szParameter, 1, 1 ) == "'"   then
do
   szParameter = substr( szParameter, 2, length( szParameter ) - 2 )
end

return szParameter
