/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
// Written by Mark Hamzy

/* Usage: tee2 [options] nmake.exe "-f makefile OPT=dbgos2c nodep" filename
**                                                               
**  Program to spawn                                         
**  Arguments to program                                
**     (all args within double quotes)                             
**  File that mirrors standard output & standard error 
**
**  Where options are:
**     -a data will be appended to filename
*/
#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_VIO
#include <os2.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <signal.h>

// Common library
#include <assrt.h>

#define SIZE_REDIRECT_BUFFER 81

/* Globals...
*/
char achBuffer[SIZE_REDIRECT_BUFFER];
char achBuffer2[SIZE_REDIRECT_BUFFER * 2];

PID  vPID                                    = 0;

/* Function prototypes...
*/
void ctrlchandler (int sig);

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
void
PrintUsage (char *pszExeName)
{
   fprintf (stderr, "Usage: tee2 [options] nmake.exe \"-f makefile OPT=dbgos2c nodep\" filename\n", pszExeName);
   fprintf (stderr, "                                                              \n");
   fprintf (stderr, " Program to spawn                                         \n");
   fprintf (stderr, " Arguments to program                                \n");
   fprintf (stderr, "    (all args within double quotes)                             \n");
   fprintf (stderr, " File that mirrors standard output & standard error \n");
   fprintf (stderr, "\n");
   fprintf (stderr, "Where options are:\n");
   fprintf (stderr, "   -a data will be appended to filename\n");
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
VOID APIENTRY
DebugFwriteCallback (PSZ pszOutput, ULONG cbLength)
{
#define FADE_SIG 0x45444146    // 'FADE'

   CHAR     szTemp[512 + sizeof (ULONG)];
   PCHAR    pchSrc;
   PCHAR    pchDest;
   ULONG    dbgStrSize = 0;

   /* Stick a FADE marker at the end of the arrays.  If the vsprintf call
   ** overwrites our sentinels then report it!
   */
   *(PULONG)((PBYTE)szTemp   + sizeof (szTemp)   - sizeof (ULONG)) = FADE_SIG;

   pchSrc  = pszOutput;
   pchDest = szTemp;

   /* change:   /r/n  to  /n
   */
   while (*pchSrc)
   {
      switch (*pchSrc)
      {
      case '\n':
         *pchDest++ = '\n';
         dbgStrSize++;
         break;

      case '\r':
         break;

      default:
         *pchDest++ = *pchSrc;
         dbgStrSize++;
         break;
      }
      pchSrc++;
   }

   *pchDest = 0;

   fwrite (szTemp, sizeof (szTemp[0]), dbgStrSize, stdout);
}

/***************************************************************************/
/*                                                                         */
/***************************************************************************/
int
main (int argc, char *argv[])
{
   FILE         *fp                     = NULL;
   char         *pszMode                = "wb";
   CHAR          achObjectBuffer[10];
   STARTDATA     StartData;
   PSZ           pszProgramSpawn        = NULL;    /* argv[n]   */
   PSZ           pszProgramSpawnArgs    = NULL;    /* argv[n+1] */
   PSZ           pszFileNameOut         = NULL;    /* argv[n+2] */
   ULONG         ulSessionID;
   HPIPE         hRPipe                 = (HPIPE)-1;
   HPIPE         hWPipe;
   HFILE         hOut;
   HFILE         hOutSave;
   HFILE         hErr;
   HFILE         hErrSave;
   ULONG         ulBytesRead;
   ULONG         ulBytesSend;
   CHAR          achQueueName[CCHMAXPATH];
   PTIB          pTIB;
   PPIB          pPIB;
   HQUEUE        pRWQueue;
   REQUESTDATA   Request;
   PBYTE         pData                  = NULL;
   BYTE          bElemPriority;
   APIRET        rc, rc2;
   ULONG         ulRet                  = 0;
   register PSZ  pszFrom;
   register PSZ  pszTo;
   register INT  i;

   // Go to stdout
   HookAssert (DebugFwriteCallback);

   /* Modify CTRL+C behavior. */
   if (signal (SIGINT, ctrlchandler) == SIG_ERR)
   {
       fprintf (stderr, "Couldn't set SIGINT\n");
       abort();
   }

#ifdef DEBUG
{
   printf ("argc = %d\n", argc);
   for (i = 0; i < argc; i++)
      printf ("argv[%d] = '%s'\n", i, argv[i]);
}
#endif

   // Read the parameters
   for (i = 1; i < argc; i++)
   {
      if (('-' == argv[i][0]) ||
          ('/' == argv[i][0])  )
      {
         static char *pszAppend    = "a";

         if (0 == strnicmp (argv[i] + 1, pszAppend, strlen (pszAppend)))
         {
            pszMode = "ab";
         }
         else
         {
            PrintUsage (argv[0]);
            goto exit;
         }
      }
      else
      {
         if (!pszProgramSpawn)
            pszProgramSpawn     = argv[i];
         else if (!pszProgramSpawnArgs)
            pszProgramSpawnArgs = argv[i];
         else if (!pszFileNameOut)
            pszFileNameOut      = argv[i];
         else
         {
            PrintUsage (argv[0]);
            goto exit;
         }
      }
   }

   if (pszFileNameOut)
   {
      fp = fopen (pszFileNameOut, pszMode);
      if (!fp)
         fprintf (stderr,
                  "Error: Could not open '%s' for writing.\n",
                  pszFileNameOut);
   }

   // boost the priority
   rc = DosSetPriority (PRTYS_PROCESS,
                        PRTYC_NOCHANGE,
                        PRTYD_MAXIMUM,
                        0);

   // Set standard output to unbuffered
   setvbuf (stdout, NULL, _IONBF, 0);

   // find out the process id
   rc = DosGetInfoBlocks (&pTIB, &pPIB);
   sprintf (achQueueName, "\\QUEUES\\tee2\\%d", pPIB->pib_ulpid);

   // Create the termination queue
   rc = DosCreateQueue (&pRWQueue, QUE_FIFO, achQueueName);

   /* Open up an anonymous pipe for communication with the child process...
   */
   rc = DosCreatePipe (&hRPipe, &hWPipe, 1024);

   hOutSave = -1;                     // Allocate a new handle
   hOut     = 1;                      // Standard handle # for stdout

   rc = DosDupHandle (hOut, &hOutSave);
   assertT (rc);

   rc = DosClose (hOut);
   assertT (rc);

   rc = DosDupHandle (hWPipe, &hOut);
   assertT (rc);

   hErrSave = -1;                     // Allocate a new handle
   hErr     = 2;                      // Standard handle # for stderr

   rc = DosDupHandle (hErr, &hErrSave);
   assertT (rc);

   rc = DosClose (hErr);
   assertT (rc);

   rc = DosDupHandle (hWPipe, &hErr);
   assertT (rc);

   StartData.Length        = sizeof (StartData);
   StartData.Related       = SSF_RELATED_CHILD;
   StartData.FgBg          = SSF_FGBG_BACK;
   StartData.TraceOpt      = SSF_TRACEOPT_NONE;
   StartData.PgmTitle      = "tee2 redirector";
   StartData.PgmName       = pszProgramSpawn;
   StartData.PgmInputs     = pszProgramSpawnArgs;
   StartData.TermQ         = achQueueName;
   StartData.Environment   = 0;
   StartData.InheritOpt    = SSF_INHERTOPT_PARENT;
   StartData.SessionType   = SSF_TYPE_DEFAULT;
   StartData.IconFile      = 0;
   StartData.PgmHandle     = 0;
   StartData.PgmControl    = SSF_CONTROL_INVISIBLE;
   StartData.InitXPos      = 0;
   StartData.InitYPos      = 0;
   StartData.Reserved      = 0;
   StartData.ObjectBuffer  = achObjectBuffer;
   StartData.ObjectBuffLen = sizeof (achObjectBuffer);

   rc2 = DosStartSession (&StartData, &ulSessionID, &vPID);
   if (NO_ERROR != rc2)
      goto exit;

   // Close the write handle to ensure notification at child termination!
   rc = DosClose (hWPipe);
   assertT (rc);

   // Restore stdout
   rc = DosDupHandle (hOutSave, &hOut);
   assertT (rc);
   rc = DosClose (hOutSave);
   assertT (rc);

   // Restore stderr
   rc = DosDupHandle (hErrSave, &hErr);
   assertT (rc);
   rc = DosClose (hErrSave);
   assertT (rc);

   if (NO_ERROR == rc2)
      do
      {
         // Fill up the read buffer
         // @TBD -- this has no timeout!  Infinite read!  Not good if an error occurs
         rc = DosRead (hRPipe,
                       (PVOID)achBuffer,
                       SIZE_REDIRECT_BUFFER - 1,
                       &ulBytesRead);

         if (NO_ERROR != rc || 0 == ulBytesRead)
            break;

         // Translate \r\r\n to \r\n
         ulBytesSend = 0;
         pszFrom     = achBuffer;
         pszTo       = achBuffer2;
         for (i = 0; i < ulBytesRead; i++)
         {
            if ('\r' == *pszFrom)
            {
               while ('\r' == *(pszFrom + 1))
               {
                  pszFrom++;
                  i++;
               }
            }

            *pszTo++ = *pszFrom++;
            ulBytesSend++;
         }

         // echo the captured output to standard output
         fwrite (achBuffer2, ulBytesSend, 1, stdout);
         // flush the output
         fflush (stdout);

         // and maybe to a file as well
         if (fp)
         {
            fwrite (achBuffer2, ulBytesSend, 1, fp);
            fflush (fp);
         }

      } while (NO_ERROR == rc);

   /* Clean up!
   */
   rc = DosClose (hRPipe);
   assertT (rc);

   // read the return code from the started session
   rc = DosReadQueue (pRWQueue,
                      &Request,
                      &ulBytesRead,
                      (PPVOID)&pData,
                      0,
                      DCWW_WAIT,
                      &bElemPriority,
                      (HEV)NULL);

   if (NO_ERROR == rc &&
       pData           )
   {
      typedef struct _TermQueue {
         USHORT id;
         USHORT rc;
      } TERMQUEUE, *PTERMQUEUE;

      PTERMQUEUE  pTQ = (PTERMQUEUE)pData;

#ifdef DEBUG
      printf ("TQ.id = %d\n", pTQ->id);
      printf ("TQ.rc = %d\n", pTQ->rc);
#endif

      // it is now our return code
      ulRet = pTQ->rc;
   }

   rc = DosFreeMem (pData);

   rc = DosCloseQueue (pRWQueue);

exit:
   return ulRet;
}

/***************************************************************************/
/* Handles SIGINT (CTRL+C) interrupt.                                      */
/***************************************************************************/
void
ctrlchandler (int sig)
{
   APIRET    rc;
   PSZ       pszMessage = "\r\n*** <Crtl>+C ***\r\n";

   /* Disallow CTRL+C during handler. */
   signal (SIGINT, SIG_IGN);

   if (vPID)
   {
      // Kill child process
      rc = DosKillProcess (DKP_PROCESS, vPID);
      assertT (rc);
   }

   // Safe writing to screen
   VioWrtTTY (pszMessage, strlen (pszMessage), 0);

   // Exit process
   abort ();
}
