/* REXX: Equivalent to xcomp
**
** Usage
**  -extract Name -release Name -family Name
**
** Relies on the followin tools:
**
**   makepath path        (md's all subdirectories for the path)
**   touch file date time (changes the timestamp for file)
**   file                 (CMVC command line program)
**   report               (CMVC command line program)
*/
SIGNAL ON HALT NAME fHalt

/* Grab the program's path */
parse source . . szPath

szCurDir = directory( )'\'
szRoot   = filespec( 'drive', szCurDir )
szRoot   = szRoot'\'word( translate( filespec( 'path', szCurDir ), ' ', '\' ), 1 )

fPrintOnly  = "0"
szComponent = ""           /* Ex: "genplib" */
szRelease   = "c.osdd"
szFamily    = "os2"

/* Read the command line parameters
*/
parse arg szRest

if szRest \= "" then
do while words( szRest ) > 0

   parse var szRest szParm szRest

   select
      when szParm = "-extract" then
      do
         parse var szRest szComponent szRest
      end
      when substr( szParm, 1, 5 ) = "-rele" then
      do
         parse var szRest szRelease szRest
      end
      when szParm = "-family" then
      do
         parse var szRest szFamily szRest
      end
      when szParm = "-printonly" then
      do
         fPrintOnly = "1"
      end
      when szParm = "-clear" then
      do
         do while lines( 'QUEUE:' ) > 0
            szReportLine = linein( 'QUEUE:' )
         end
         exit
      end
      when szParm = "-currentdirectory" then
      do
         szRoot = directory( )
      end
      otherwise
      do
         call fSayUsage

         say "Unknown option" szParm
         exit
      end
   end
end

if szComponent = "" then
do
   szComponent = "$$$NONE$$$"
end

if szRelease   = "" |,
   szFamily    = ""    then
do
   call fSayUsage

   if szRelease   = "" then say "'-release ReleaseName' must be given"
   if szFamily    = "" then say "family name is empty!"
   exit
end

/*
** Sample output for internal stuff
**

Report -raw -view FileView -where "releaseName in ('c.osdd') and compName in ('genplib')"

genplib.def|c.osdd|genplib|1.1|1995/04/21 18:54:32|1997/12/08 14:44:38|1997/12/1
2 15:08:31|src/osdd/genplib/genplib.def|1.1|1995/04/21 18:54:32|1997/12/08 14:44
:38|src/osdd/genplib/genplib.def||0640||||||||||||||||||||||||||||||||||||||||||
||||||||

                                                 new drop date
basename  rele   comp    com  new add date        last update         New Path Name
genplib.h|c.osdd|genplib|1.9|1995/04/21 18:54:35||1998/01/22 15:16:18|src/osdd/g
                 curr                                                 locked mode    nls
enplib/genplib.h|1.10|1995/04/21 18:54:35||src/osdd/genplib/genplib.h|      |0640|||| |
|||||||||||||||||||||||||||||||||||||||||||||

genplibp.h|c.osdd|genplib|1.2|1995/04/21 18:54:39||1998/01/22 15:35:21|src/osdd/
genplib/genplibp.h|1.2|1995/04/21 18:54:39|1998/01/22 15:35:21|src/osdd/genplib/
genplibp.h||0640||||||||||||||||||||||||||||||||||||||||||||||||||

genpwp.def|c.osdd|genplib|1.1|1995/04/21 18:54:43|1997/12/08 14:44:40|1997/12/12
 15:08:31|src/osdd/genplib/genpwp.def|1.1|1995/04/21 18:54:43|1997/12/08 14:44:4
0|src/osdd/genplib/genpwp.def||0640|||||||||||||||||||||||||||||||||||||||||||||
|||||

gplassrt.c|c.osdd|genplib|1.7|1995/04/21 18:54:47||1998/01/22 15:35:25|src/osdd/
genplib/gplassrt.c|1.7|1995/04/21 18:54:47|1998/01/22 15:35:25|src/osdd/genplib/
gplassrt.c||0640||||||||||||||||||||||||||||||||||||||||||||||||||

New Path Name                  Release  Component    Current  Committed Locked by     Mode  New Add Date        New Drop Date       Last Update         nlsKey baseVID stateLogin State
src/osdd/genplib/genplib.def   c.osdd   genplib      1.1      1.1                     0640  1995/04/21 18:54:32 1997/12/08 14:44:38 1997/12/12 15:08:31
src/osdd/genplib/genplib.h     c.osdd   genplib      1.10     1.9                     0640  1995/04/21 18:54:35                     1998/01/22 15:16:18
src/osdd/genplib/genplibp.h    c.osdd   genplib      1.2      1.2                     0640  1995/04/21 18:54:39 1998/01/22 15:35:21 1998/01/22 15:35:21
src/osdd/genplib/genpwp.def    c.osdd   genplib      1.1      1.1                     0640  1995/04/21 18:54:43 1997/12/08 14:44:40 1997/12/12 15:08:31
src/osdd/genplib/gplassrt.c    c.osdd   genplib      1.7      1.7                     0640  1995/04/21 18:54:47 1998/01/22 15:35:25 1998/01/22 15:35:25

*/

aFileViews.0 = 0
iFileCount   = 0

call fGenerateFileViewReport szComponent szRelease iFileCount

if fPrintOnly then
do
   do i = 1 to aFileViews.0
      call fPrintFile i
   end

   exit
end

do i = 1 to aFileViews.0

   szFullFileName = translate( szRoot'/'aFileViews.i.!szNewPathName, '\', '/' )

   szFullDirectory = filespec( 'drive', szFullFileName ) || filespec( 'path', szFullFileName )
   szFullDirectory = left( szFullDirectory, length( szFullDirectory ) - 1 )

   szNewDir = directory( szFullDirectory )

   if szNewDir = "" then
   do
      '@makepath 'szFullDirectory' > nul 2>&1'
      szNewDir = directory( szFullDirectory )
   end

   if szNewDir \= "" then
   do
      say "Extracting" aFileViews.i.!szNewPathName "to" szFullDirectory"\"aFileViews.i.!szBaseName
      '@file -extract 'aFileViews.i.!szNewPathName' -release 'szRelease
      if rc = 1 then
      do
         say 'File' aFileViews.i.!szNewPathName 'not extracted ( rc = 'rc' )'
      end
      else if rc = 0 then
      do
         '@attrib -r -s -h' aFileViews.i.!szBaseName' > nul 2>&1'
         parse var aFileViews.i.!szLastUpdate szYear '/' szMonth '/' szDay szHours ':' szMinutes ':' szSeconds
         szYear = right( szYear, 2 )
         '@touch 'aFileViews.i.!szBaseName szMonth'/'szDay'/'szYear szHours':'szMinutes':'szSeconds' > nul 2>&1'
      end
      else if rc = 99 then
         /* <Ctrl>+C hit! */
         nop
      else
      do
         say "Unknown ( rc = "rc" )"
      end
   end
end

szNewDir = directory( szCurDir )

exit


/*-----------------------------------Functions-follow:--------------------------------------------------*/


/*********************************************************************************************************/
/*                                                                                                       */
/* Print out a report line that is properly formated                                                     */
/*                                                                                                       */
/*********************************************************************************************************/
fPrintDefect: parse arg iDefectNum

aDefects.iDefectNum.!bPrinted = 1

szOut =           left( aDefects.iDefectNum.!szDefect,     6 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szComponent, 15 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szState,      8 ) || ' '
/*
szOut = szOut ||  left( aDefects.iDefectNum.!szOrigin,     8 ) || ' '
*/
szOut = szOut ||  left( aDefects.iDefectNum.!szOwner,      8 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szSeverity,   3 ) || ' '
szOut = szOut || right( aDefects.iDefectNum.!szAge,        3 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szPriority,   4 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szAfld1,      4 ) || ' '
szOut = szOut ||  left( aDefects.iDefectNum.!szTarget,     6 ) || ' '
szOut = szOut || aDefects.iDefectNum.!szTitle

say szOut

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Print out a report line that is properly formated                                                     */
/*                                                                                                       */
/*********************************************************************************************************/
fPrintFile: parse arg iFileNum

aFileViews.iFileNum.!bPrinted = 1

szOut =           left( aFileViews.iFileNum.!szRelease,      8 ) || ' '
szOut = szOut ||  left( aFileViews.iFileNum.!szComponent,   14 ) || ' '
szOut = szOut ||  left( aFileViews.iFileNum.!szNewPathName, 40 ) || ' '
szOut = szOut ||  left( aFileViews.iFileNum.!szLastUpdate,  21 ) || ' '

say szOut

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Call CMVC for the list of defects given                                                               */
/*                                                                                                       */
/*********************************************************************************************************/
fGenerateDefectReport: parse arg szDefectList iMaxDefects c

/* Generate and queue a CMVC report query
*/
rc = lineout( "STDERR:", "Read report ("iMaxDefects+1"-"iMaxDefects+c "of" aList.0") on defect" szDefectList )
'@Report -raw -view DefectView -where "name in ('szDefectList')" | rxqueue'

/* Read the report from the queue
*/
do while lines( 'QUEUE:' ) > 0

   szReportLine = linein( 'QUEUE:' )

   iMaxDefects = iMaxDefects + 1

   call fParseDefectReport iMaxDefects aDefects szReportLine

end

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Call CMVC for the track given                                                                         */
/*                                                                                                       */
/*********************************************************************************************************/
fGenerateTrackReport: parse arg szTrack iMaxTrack

/* Generate and queue a CMVC report query
*/
rc = lineout( "STDERR:", "Read track report (" szTrack ")" )

szTrackQuote = "'"szTrack"'"
'@Report -raw -view TrackView -where "defectName in ('szTrackQuote')" | rxqueue'

/* Read the report from the queue
*/
do while lines( 'QUEUE:' ) > 0

   szReportLine = linein( 'QUEUE:' )

   iMaxTrack = iMaxTrack + 1

   call fParseTrackReport iMaxTrack aTracks szReportLine

end

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Call CMVC for the track given to get levelmem info                                                    */
/*                                                                                                       */
/*********************************************************************************************************/
fGenerateLevelMemReport: parse arg szTrack iMaxLevelMem

/* Generate and queue a CMVC report query
*/
rc = lineout( "STDERR:", "Read level member report (" szTrack ")" )

szTrackQuote = "'"szTrack"'"
'@Report -raw -view LevelmemberView -where "defectName in ('szTrackQuote')" | rxqueue'

/* Read the report from the queue
*/
do while lines( 'QUEUE:' ) > 0

   szReportLine = linein( 'QUEUE:' )

   iMaxLevelMem = iMaxLevelMem + 1

   call fParseLevelMemReport iMaxLevelMem aLevelMems szReportLine

end

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Call CMVC for the component given to get fileview info                                                    */
/*                                                                                                       */
/*********************************************************************************************************/
fGenerateFileViewReport: parse arg szComponent szRelease iMaxFileView

/* Generate and queue a CMVC report query
*/
szComponentQuote = "'"szComponent"'"
szReleaseQuote   = "'"szRelease"'"

/* Build it! */
szCommand = '@Report -raw -view FileView -where "releaseName in ('szReleaseQuote')'

if szComponent \= "$$$NONE$$$" then
   szCommand = szCommand || ' and compName in ('szComponentQuote')"'
else
   szCommand = szCommand || '"'

szCommand = szCommand || ' | rxqueue'

/* Call it! */
say "Running $$$"szCommand"$$$"
szCommand

if 0 >= lines( 'QUEUE:' ) then
do
   szSayLine = "No files reported for release" szRelease

   if szComponent \= "$$$NONE$$$" then
      szSayLine = szSayLine "and component" szComponent

   say szSayLine

   return
end

szSayLine = "Read file view report"

if szComponent \= "$$$NONE$$$" then
   szSayLine = szSayLine "component = (" szComponent ")"

szSayLine = szSayLine "release = (" szRelease ")"

rc = lineout( "STDERR:", szSayLine )

/* Read the report from the queue
*/
do while lines( 'QUEUE:' ) > 0

   szReportLine = linein( 'QUEUE:' )

   iMaxFileView = iMaxFileView + 1

   call fParseFileViewReport iMaxFileView aFileViews szReportLine

end

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Parse a raw report line into a stem variable for internal usage                                       */
/*                                                                                                       */
/*********************************************************************************************************/
fParseDefectReport: parse arg iDefectNum aDefects szReportLine

/* Parse the raw report line
*/
parse var szReportLine . '|' szDefect '|' szComponent '|' szRest
parse var szRest       szRelease '|' szOwner '|' szState '|' szRest
parse var szRest       szAnswer '|' szSeverity '|' szTitle '|' szRest
parse var szRest       szAge '|' . '|' . '|' . '|' szLastUpdate '|' szRest
parse var szRest       szAddDate '|' szAssignDate '|' szResponseDate '|' szRest
parse var szRest       . '|' szOwnerName '|' szOwnerDept '|' szRest
parse var szRest       . '|' szOrigin '|' szOriginName '|' szRest
parse var szRest       szOriginDept '|'  szSymptom '|' szRest
parse var szRest       szPhaseFound '|' . '|' szPriority '|' szTarget '|' szRest
parse var szRest       szApar '|' szAfld1 '|' szAfld2 '|' szRest

/* Add it to the array of defects
*/
aDefects.0 = iDefectNum

aDefects.iDefectNum.!bPrinted      = 0
aDefects.iDefectNum.!szDefect      = szDefect
aDefects.iDefectNum.!szComponent   = szComponent
aDefects.iDefectNum.!szRelease     = szRelease
aDefects.iDefectNum.!szOwner       = szOwner
aDefects.iDefectNum.!szState       = szState
aDefects.iDefectNum.!szAnswer      = szAnswer
aDefects.iDefectNum.!szSeverity    = szSeverity
aDefects.iDefectNum.!szTitle       = szTitle
aDefects.iDefectNum.!szAge         = szAge
aDefects.iDefectNum.!szLastUpdate  = szLastUpdate
aDefects.iDefectNum.!szAddDate     = szAddDate
aDefects.iDefectNum.!szAssignDate  = szAssignDate
aDefects.iDefectNum.!szResponseDate= szResponseDate
aDefects.iDefectNum.!szOwnerName   = szOwnerName
aDefects.iDefectNum.!szOwnerDept   = szOwnerDept
aDefects.iDefectNum.!szOrigin      = szOrigin
aDefects.iDefectNum.!szOriginName  = szOriginName
aDefects.iDefectNum.!szOriginDept  = szOriginDept
aDefects.iDefectNum.!szSymptom     = szSymptom
aDefects.iDefectNum.!szPhaseFound  = szPhaseFound
aDefects.iDefectNum.!szPriority    = szPriority
aDefects.iDefectNum.!szTarget      = szTarget
aDefects.iDefectNum.!szApar        = szApar
aDefects.iDefectNum.!szAfld1       = szAfld1
aDefects.iDefectNum.!szAfld2       = szAfld2

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Parse a raw report line into a stem variable for internal usage                                       */
/*                                                                                                       */
/*********************************************************************************************************/
fParseTrackReport: parse arg iCurrentTrack aTracks szReportLine

/* Parse the raw report line
*/
parse var szReportLine szRelease '|' szDefect '|' . '|' szRest
parse var szRest       szState '|' . '|' szAddDate '|' szRest
parse var szRest       szUserLogin '|' szUserName '|' szUserDept '|' szRest
parse var szRest       . '|' szLastUpdate '|' szPrefix '|' szTitle

/* Add it to the array of defects
*/
aTracks.0 = iCurrentTrack

aTracks.iCurrentTrack.!bPrinted      = 0
aTracks.iCurrentTrack.!szDefect      = szDefect
aTracks.iCurrentTrack.!szRelease     = szRelease
aTracks.iCurrentTrack.!szState       = szState
aTracks.iCurrentTrack.!szAddDate     = szAddDate
aTracks.iCurrentTrack.!szUserLogin   = szUserLogin
aTracks.iCurrentTrack.!szUserName    = szUserName
aTracks.iCurrentTrack.!szUserDept    = szUserDept
aTracks.iCurrentTrack.!szLastUpdate  = szLastUpdate
aTracks.iCurrentTrack.!szPrefix      = szPrefix

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Parse a raw report line into a stem variable for internal usage                                       */
/*                                                                                                       */
/*********************************************************************************************************/
fParseLevelMemReport: parse arg iCurrentTrack aLevelMems szReportLine

/* Parse the raw report line
*/
parse var szReportLine szLevel '|' szRelease '|' szDefect '|' szRest
parse var szRest       . '|' szUserLogin '|' szUserName '|' szRest
parse var szRest       szUserDept '|' szPrefix

/* Add it to the array of defects
*/
aLevelMems.0 = iCurrentTrack

aLevelMems.iCurrentTrack.!bPrinted      = 0
aLevelMems.iCurrentTrack.!szLevel       = szLevel
aLevelMems.iCurrentTrack.!szRelease     = szRelease
aLevelMems.iCurrentTrack.!szDefect      = szDefect
aLevelMems.iCurrentTrack.!szUserLogin   = szUserLogin
aLevelMems.iCurrentTrack.!szUserName    = szUserName
aLevelMems.iCurrentTrack.!szUserDept    = szUserDept
aLevelMems.iCurrentTrack.!szPrefix      = szPrefix

return

/*********************************************************************************************************/
/*                                                                                                       */
/* Parse a raw report line into a stem variable for internal usage                                       */
/*                                                                                                       */
/*********************************************************************************************************/
fParseFileViewReport: parse arg iCurrentFile aFileViews szReportLine

/* Parse the raw report line
*/
parse var szReportLine szBaseName '|' szRelease '|' szComponent '|' szRest
parse var szRest       szCommitted '|' szNewAddDate '|' szNewDropDate '|' szRest
parse var szRest       szLastUpdate '|' szNewPathName '|' szCurrent '|' . '|' szRest
parse var szRest       . '|' . '|' szLocked '|' szMode '|' . '|' . '|' . '|' szRest
parse var szRest       szNlsKey '|' .

/* Add it to the array of defects
*/
aFileViews.0 = iCurrentFile

aFileViews.iCurrentFile.!bPrinted      = 0
aFileViews.iCurrentFile.!szBaseName    = szBaseName
aFileViews.iCurrentFile.!szRelease     = szRelease
aFileViews.iCurrentFile.!szComponent   = szComponent
aFileViews.iCurrentFile.!szCommitted   = szCommitted
aFileViews.iCurrentFile.!szNewAddDate  = szNewAddDate
aFileViews.iCurrentFile.!szNewDropDate = szNewDropDate
aFileViews.iCurrentFile.!szLastUpdate  = szLastUpdate
aFileViews.iCurrentFile.!szNewPathName = szNewPathName
aFileViews.iCurrentFile.!szCurrent     = szCurrent
aFileViews.iCurrentFile.!szLocked      = szLocked
aFileViews.iCurrentFile.!szMode        = szMode
aFileViews.iCurrentFile.!szNlsKey      = szNlsKey

return

/*********************************************************************************************************/
/*                                                                                                       */
/*********************************************************************************************************/
fSayUsage:

parse source . . szPath .

say "Usage "filespec( 'name', szPath )
say " -extract ComponentName [-release ReleaseName] [-family FamilyName]"
say
say " where: release defaults to c.osdd and family defaults to os2"
say

return

/*********************************************************************************************************/
/*                                                                                                       */
/*********************************************************************************************************/
fHalt:
say "Ctrl-Break hit!"

if lines( 'QUEUE:' ) > 1 then
   say "Flushing external queue..."
/* Flush the external queue */
do while lines( 'QUEUE:' ) > 1
   szLine = linein( 'QUEUE:' )
end
