/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/************************************************************************
 *
 * SOURCE FILE NAME = BOOKLET.H
 *
 * DESCRIPTION = Public output Booklet header
 *
 *
 * VERSION = V1.0
 * DATE    = 09/29/98
 * AUTHOR  = Arsenij Dorosevich
 *
 *
 * NOTES
 *
 * STRUCTURES
 *
 * EXTERNAL REFERENCES
 *
 * EXTERNAL FUNCTIONS
 *
*/
#ifndef _gpl_booklet_h
#define _gpl_booklet_h

#define INCL_GENPLIB_THREAD
#include <genplib.h>
#include "nup.h"

/*****************************************************************************\
** DEFINES                                                                   **
\*****************************************************************************/

//
// Booklet initialization flags
//
#define BOOKLET_NUP         0x01
#define BOOKLET_EJECT       0x02
#define BOOKLET_OMNI        0x04
#define BOOKLET_LANDSCAPE   0x08
#define BOOKLET_DUPLEX      0x10
#define BOOKLET_UPPERTRAY   0x20
#define BOOKLET_BOOKLET     0x40    
#define BOOKLET_UNCOLLATE   0x80    


//
// Definition for pages 
//
#define BOOKLET_FIRST_LOGICAL_PAGE           1
#define BOOKLET_SECOND_LOGICAL_PAGE          2


//
// Definitions for Booklet's output state
//
#define  BOOKLET_OUTPUT_DIRECT               0
#define  BOOKLET_OUTPUT_SETUP                1
#define  BOOKLET_OUTPUT_EJECT                2
#define  BOOKLET_OUTPUT_NUP1                 3
#define  BOOKLET_OUTPUT_NUP2                 4
#define  BOOKLET_OUTPUT_FONTS                5
#define  BOOKLET_OUTPUT_IGNORE               6
#define  BOOKLET_OUTPUT_BLANKPAGE            7
#define  BOOKLET_OUTPUT_ENDDOC               8  


//
// Definitions for thread's output state
//
#define  BOOKLET_THREAD_DIRECT                          0
#define  BOOKLET_THREAD_BUFFER                          1


//
// Size of allocated memory
//
#define  BOOKLET_ALLOC_SIZE                             8192
#define  BOOKLET_PAGE_ALLOC_SIZE                        1048576
#define  BOOKLET_BUFFER_SIZE                            4096
#define  BOOKLET_BLANKPAGE_SIZE                         1024

/*****************************************************************************\
** STRUCTURES                                                                **
\*****************************************************************************/

typedef struct _BOOKLETSETUP  BOOKLETSETUP, *PBOOKLETSETUP;

struct _BOOKLETSETUP
{
   PBOOKLETSETUP        pNextSetup; // Ptr to next setup
   PBYTE                pbSetup;    // 
   ULONG                ulSetupLenght; //
};


typedef struct _BOOKLETPAGES  BOOKLETPAGES, *PBOOKLETPAGES;

struct _BOOKLETPAGES
{
   PBOOKLETPAGES        pNextPage;       // Ptr to next page
   ULONG                ulPagePosition;  // Page's start position
   USHORT               usSetup;         // 
   BOOL                 fPrint;          // 
};


typedef struct  _BOOKLET /* This struct will addet to _DV struct */
{
   PBYTE                pbFontsBuffer; // Buffer for fonts storing
   ULONG                ulFontsBufferPos; // Fonts buffer pos
   ULONG                ulFontsLenght; // Fonts buffer lenght
   HFILE                hFontsFile; // Temporary file header for fonts storing

   PBYTE                pbPagesBuffer; // Buffer for
   ULONG                ulPagesBufferPos; // Current position in Page Buffer
   ULONG                ulOutputLenght; // Size of all output that went through thread
   HFILE                hPagesFile;  // Temporary file header for pages storing
   PBOOKLETPAGES        pPages; // Ptr to list of pages

   PBYTE                pbSetup; // Buffer for "SetUp" command sequence
   ULONG                ulSetupLenght; // Size of "SetUp" command sequence

   PBYTE                pbEject; // Buffer for "Eject" command sequence
   ULONG                ulEjectLenght; // Size of "Eject" command sequence

   PBYTE                pbNup1; // Buffer for Nup's sequence for 1st logical page
   ULONG                ulNup1Lenght; // Size of Nup's sequence for 1st logical page

   PBYTE                pbNup2; // Buffer for Nup's sequence for 2nd logical page
   ULONG                ulNup2Lenght; // Size of Nup's sequence for 2nd logical page

   PBYTE                pbBlankPage; // Blank page for Omni
   ULONG                ulBlankPageLenght; // Size of buffer for blank page

   PBYTE                pbEnddoc; 
   ULONG                ulEnddocLenght; 

   PBOOKLETSETUP        pSetups; // Ptr to list of setups

   USHORT               usPrevChannelState; // Previous state of function, that write to thread
   USHORT               usChannelState; // Current state of function, that write to thread
   USHORT               usThreadState;  // Current state of Thread

   PSZ                  pszRangeOfPages;
   PSZ                  pszDocumentName;
   PSZ                  pszPagesFile;
   PSZ                  pszFontsFile;
   BYTE                 bFlags;

   ULONG                ulPagesNum;     // Page's counter
   USHORT               usSetupNum;     // Setup's counter 
   BOOL                 fPagePrinted;

   ULONG                ulCopies;       

} BOOKLET, *PBOOKLET;

/*****************************************************************************\
** FUNCTION PROTOTYPES                                                       **
\*****************************************************************************/
extern BOOL APIENTRY GplBookletCheckOutput( HTHREAD, USHORT );
extern BOOL APIENTRY GplBookletCheckThread( HTHREAD , USHORT );
extern VOID APIENTRY GplBookletDestroy( HTHREAD  );
extern BOOL APIENTRY GplBookletEnabled( HTHREAD );
extern VOID APIENTRY GplBookletIncOutputLenght ( HTHREAD , ULONG );
extern VOID APIENTRY GplBookletInit ( HTHREAD , HMODULE , PSZ , PSZ , BYTE , ULONG );  
extern VOID APIENTRY GplBookletMarkPages( PBOOKLET , ULONG , BOOL );
extern VOID APIENTRY GplBookletNewPage ( HTHREAD );
extern BOOL APIENTRY GplBookletOpenFile ( PHFILE , PSZ );
extern VOID APIENTRY GplBookletRangeOfPages( PBOOKLET );
extern VOID APIENTRY GplBookletReplayOutput ( HTHREAD );
extern VOID APIENTRY GplBookletResetOutput ( HTHREAD );
extern VOID APIENTRY GplBookletResetPage( HTHREAD );
extern VOID APIENTRY GplBookletSaveEject ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSaveFonts  ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSaveBlankPage ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSaveEnddoc ( HTHREAD , PBYTE , ULONG );    
extern VOID APIENTRY GplBookletSaveNup1 ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSaveNup2 ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSavePages ( HTHREAD , PBYTE , ULONG );
extern VOID APIENTRY GplBookletSaveSetup ( HTHREAD , PBYTE , ULONG );
extern BOOL APIENTRY GplBookletSendFonts ( HTHREAD );
extern BOOL APIENTRY GplBookletSendPage ( HTHREAD , ULONG , USHORT );
extern BOOL APIENTRY GplBookletSendSetup( HTHREAD, USHORT );
extern VOID APIENTRY GplBookletSetOutput( HTHREAD , USHORT );
extern VOID APIENTRY GplBookletSetThread  ( HTHREAD , USHORT );
extern VOID APIENTRY GplBookletSwitchPaper( HTHREAD );
extern VOID APIENTRY GplBookletToBuffer( HTHREAD , PBYTE , ULONG );
extern BOOL APIENTRY GplBookletWriteToFile( HFILE, PBYTE, PULONG );


#endif
