/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = DBCS.H
 *
 * DESCRIPTION = This module contains all MACROS, IDs, constants and
 *               data type definitions used by the NLS/DBCS functions
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 03-22-96
 * AUTHOR  = IBM Japan, IBM USA
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_dbcs_h
#define _gpl_dbcs_h

/*===========================================================================*
 |      The Return Code for General Convert Routines
 *===========================================================================*/

#define GPL_DBCS_NOERROR                NO_ERROR
#define GPL_DBCS_CONV_UNKNOWNTYPE       0x1000
#define GPL_DBCS_CONV_BUFOVERRUN        0x1001
#define GPL_DBCS_NOTCONVERTED           0x1002
#define GPL_DBCS_CONV_USERDEFINED       0x1003
#define GPL_DBCS_INVALID_CODEPAGE       0x1010
#define GPL_DBCS_NOTIMPLEMENTED         0x1011
#define GPL_DBCS_DOSAPI_ERROR           0x1012
#define GPL_DBCS_DBCSENV_INVALID        0x1020
#define GPL_DBCS_USERENV_INVALID        0x1021
#define GPL_DBCS_ERR_MEMORY_ALLOC       0x1030
#define GPL_DBCS_ERR_MEMORY_FREE        0x1031

#define COUNTRY_JAPAN                   81
#define CODEPAGE_JAPAN                  932
#define CODEPAGE_JAPAN_SAA              942
#define CODEPAGE_JAPAN_NEW              943

#define COUNTRY_TAIWAN                  88              // <TWN>
#define CODEPAGE_TAIWAN                 950             // <TWN>

/*****************************************************************************/
/* Character Set Definitions:                                                */
/*                                                                           */
/*****************************************************************************/
#define CHAR_SET_NOT_DEFINED            0L  //For non Japanese case
#define CHAR_SET_JIS_C_6226_1978        1L  //JIS 78 character set
#define CHAR_SET_JIS_X_0208_1983        2L  //JIS 83 character set
#define CHAR_SET_JIS_X_0208_1990        3L  //JIS 90 character set

/*****************************************************************************/
/* Character Set Extension Definitions:                                      */
/*                                                                           */
/*****************************************************************************/
#define CHAR_SET_EXTENSION_IBM_KANJI    0x00000001L
#define CHAR_SET_EXTENSION_NEC_KANJI    0x00000002L

/*****************************************************************************/
/* Character Encoding Method Definitions:                                    */
/*                                                                           */
/*****************************************************************************/
#define CHAR_ENCODING_SHIFT_JIS             1L
#define CHAR_ENCODING_JIS                   2L
#define CHAR_ENCODING_EUC_PACKED_FORMAT     3L
#define CHAR_ENCODING_EUC_TWO_BYTE_FORMAT   4L
#define CHAR_ENCODING_BIG5                  5L                         // <TWN>
#define CHAR_ENCODING_5550                  6L                         // <TWN>

/*****************************************************************************/
/* Misc Definitions                                                          */
/*****************************************************************************/
#define MAXRANGELEN                         12

/*****************************************************************************/ 
/* Macro Definitions                                                         */ 
/*****************************************************************************/ 
#define GplIsALNUM(x, p)  (((UCHAR)x >= 0x20)   && ((UCHAR)x < *p))             
#define GplIsKANA( x, p)  (((UCHAR)x  > *(p+1)) && ((UCHAR)x < *(p+2)))         
#define GplIsOTHER(x, p)  (((UCHAR)x  > *(p+3)) && ((UCHAR)x != 0  ))           
#define GplIsDBCS( x, p)  (!GplIsALNUM(x,p) && !GplIsKANA(x,p) && !GplIsOTHER(x,p)) 

///*****************************************************************************/
///*      The Mode Command For Dealing DBCS Mode, such as Kanji in, Kanji out. */
///*****************************************************************************/
//typedef struct _MODECOMMAND{
//    PSZ     pszModeIn;
//    ULONG   ulModeInLen;
//    PSZ     pszModeOut;
//    ULONG   ulModeOutLen;
//} MODECMD;
//
//typedef MODECMD *PMODECMD;

/*****************************************************************************/
/*      The Code Page Range Information Table Structure.                     */
/*****************************************************************************/
typedef struct _CPRANGEINFO{
    ULONG   ulCodePage;
    BYTE    abDbcsRange[MAXRANGELEN];
    BYTE    abUserRange[MAXRANGELEN];
} CPRANGEINFO;

typedef struct _STRINOUT{

    PSZ         pszStringIn;            // Pointer to string before conv
    ULONG       ulStrInLen;             // String length
    PSZ         pszStringOut;           // Pointer to string after conv
    ULONG       ulStrOutLen;            // Converted string length
    ULONG       ulInCharSet;            // Character Set Uses By System
    ULONG       ulInCharSetExtension;   // Character Set Ext Uses By System
    ULONG       ulInCharEncoding;       // Character Encoding Uses By System
    ULONG       ulOutCharSet;           // Character Set Uses By Device
    ULONG       ulOutCharSetExtension;  // Character Set Ext Uses By Device
    ULONG       ulOutCharEncoding;      // Character Encoding Uses By Device
    CHAR        abModeIn[32];           //
    ULONG       ulModeInLen;            //
    CHAR        abModeOut[32];          //
    ULONG       ulModeOutLen;           //
    ULONG       ulCodePage;             // Cope page Information Save Area
    CHAR        abDBCSEnv[MAXRANGELEN]; // DBCS Code Range Information Array
    CHAR        abUSEREnv[MAXRANGELEN]; // USER Range Information Array
    ULONG       fConvertOption;         

}   STRINOUT;

typedef STRINOUT    *PSTRINOUT, *HSTRINOUT, **PHSTRINOUT;

#define FCO_ANKTOJIS        0x00010000                                          
#define FCO_ANKTOSJIS       0x00020000                                          
#define FCO_RETURNUDC       0x00040000                                          

/*===========================================================================*
 |      The Functions Prototype Declaration
 *===========================================================================*/
APIRET APIENTRY   GplDBCSConvertCreateInstance( HMCB, PHSTRINOUT );
APIRET APIENTRY   GplDBCSConvertDeleteInstance( HSTRINOUT );
APIRET APIENTRY   GplDBCSCodeConvert( HSTRINOUT );
APIRET APIENTRY   GplDBCSGetDBCSRange( ULONG, ULONG, ULONG, PBYTE, ULONG, PBYTE);
APIRET APIENTRY   GplNLSQuerySystemCharSetID( ULONG, PULONG );
ULONG  APIENTRY   GplDBCSJpnJisSwap( ULONG);                                    
BOOL   APIENTRY   GplDBCSGetUDCImage( HMCB, ULONG, ULONG, ULONG, PBYTE);        
BOOL   APIENTRY   GplDBCSScaleImg( PBYTE, POINTL, PBYTE, POINTL);               
PSZ    APIENTRY   GplDBCSFindUDC( PSZ, PULONG, PCHAR, PCHAR);                   
BOOL   APIENTRY   GplDBCSisUDC( ULONG, PCHAR);                                  

#endif
