/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = DITHER.H
 *
 * DESCRIPTION = dither.h contains typedefs, defines, and function
 *               definitions for dither.c and routines that use the
 *               dither API's:
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 11-28-94
 * AUTHOR  =
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_dither_h
#define _gpl_dither_h

/****************************************************************************/
/* The purpose of a Dither function is to convert picture pels in sizes     */
/* 2, 4, 8, 16, 24, or 32 bits of RGB/K color to a 3 or 4 bitpel consisting */
/* of Cyan Magenta Yellow and optionally K(Black).  The best dither         */
/* functions diffuse or spread out colors and also acting as a low pass     */
/* filter to changes in color.  This diffusion of color causes              */
/* a proper density of Cyan Magenta and Yellow such that the eye percieves  */
/* the correct color at a distance, for example orange or light pink.       */
/*                                                                          */
/* Percievable print spectrum is less than a display device, and in the     */
/* future, printers will use 7 inks, CMYKRGB in order to widen the spectrum.*/
/* For this reason and in order to ease enhanceability, parameters          */
/* have been put in structures rather than enumerated in the function       */
/* interface.                                                               */
/*                                                                          */
/* In order to optomize throughput, a dither handle                         */
/* is initialized when GplDitherCreateInstance is invoked.                  */
/* This includes allocation                                                 */
/* and initialization of transform tables and parameters.                   */
/*                                                                          */
/*   GplDitherCreateInstance(PHDITHER       ppdh,                           */
/*                 HMCB           hmcb,                                     */
/*                 PDITHEREQUEST  pReq );                                   */
/*            "ppdh" is a pointer to a pointer to the dither handler        */
/*                    that created by GplDitherCreateInstance.              */
/*            "hmcb" if not zero is a handler pointer to the API heap.      */
/*            "pReq" is a pointer to the API's pre-stuffed request          */
/*                      structure                                           */
/*                   The following fields MUST be set by the API:           */
/*                       bReq                                               */
/*                       lDitherType                                        */
/*                       snap                                               */
/*                       pSrcInfo                                           */
/*                          REQUIRES :                                      */
/*                              cx                                          */
/*                              cy                                          */
/*                       ulColorTech                                        */
/*                       ulBufferWidth                                      */
/*                       ulGamma                                            */
/*                       lBias                                              */
/*                                                                          */
/*                  The following optional elements are used if not zero:   */
/*                       When lDitherType == HT_ERR_DIFFUSION               */
/*                          Set SrcOptions using the macro                  */
/*                              GPL_SETUP_DITHER( Req, ulLevel)             */
/*                  If lDitherType == MAGIC_SQUARES                         */
/*                       lHue;                                              */
/*                       lSaturation;                                       */
/*                       lValue;                                            */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/* Dithering is actually performed by GplDitherRGBtoCMYK.                   */
/*                                                                          */
/*   GplDitherRGBtoCMYK( PHDITHER        ppdh,                              */
/*                       PDITHEREQUEST   pReq);                             */
/*                                                                          */
/*                   The following fields MUST be set by the API:           */
/*                      fNewColors    MUST be been set TRUE at beginning    */
/*                                     of page!!                            */
/*                               This element is set to FALSE on return.    */
/*                    if fNewColors == TRUE:                                */
/*                       ulGamma                                            */
/*                       lBias                                              */
/*                                                                          */
/*                                                                          */
/*                   if a diffusion dither algorithm is selected,           */
/*                       use the macro:                                     */
/*                       GPL_SETUP_DITHER( Req, ulLevel)                    */
/*                               to setup the SrcOptions structure          */
/*                   else if Magic Squares or Ordered Squares is selected   */
/*                       the following 3 fields must be setup:              */
/*                       lHue                                               */
/*                       lSaturation                                        */
/*                       lValue                                             */
/*                                                                          */
/*                                                                          */
/*                       pSrcInfo Must have the following two fields set    */
/*                            cx                                            */
/*                            cy                                            */
/*                    if BitsPerPel < 16 and                                */
/*                       fNewColors == TRUE  then                           */
/*                       the the pSrcInfo array, argbColor[256] must be     */
/*                            initialized                                   */
/*                                                                          */
/*                                                                          */
/*                       pbSrc  pointer to the surface bitmap               */
/*                       pbDest pointer to a destination buffer wherein     */
/*                              each output color is separated into a block */
/*                              beginning with Black, Cyan, Magenta, Yellow */
/*                              If Black is not requested, Cyan is at the   */
/*                              beginning of the buffer.                    */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*  GplDitherRGBtoRGB is simply a shell for GplDitherRGBtoCMYK.  It simply  */
/*  inverts the CMY pDest buffer to RGB for a printer that requires RGB.    */
/*                                                                          */
/*   GplDitherRGBtoRGB(  PHDITHER      ppdh,                                */
/*                       PDITHEREQUEST   pReq);                             */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*  GplDitherDeleteInstance must be called at the end of a document so      */
/*  that memory that was allocated in GplDitherCreateInstance may be freed. */
/*                                                                          */
/*   GplDitherDeleteInstance(     HDITHER   pdh,                            */
/*                       PDITHEREQUEST);                                    */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*  Sepcific printer paper, ink, resolution, and other characteristics cause*/
/*  the actual color printed to be skewed relative to the actual color      */
/*  desired.  This skew occurs over the entire color gamut in a non-linear  */
/*  fashion.  In order to approximate for the non-linear skew, a gamma      */
/*  compensation curve is used for each color.  Since colors behave         */
/*  differently because of their different reflected frequencies, the       */
/*  user is allowed to modify each colors gamma.                            */
/*                                                                          */
/*  Increasing gamma's will lighten the image which would look too dark     */
/*  without it.                                                             */
/*                                                                          */
/*  The following two functions are call in the above dither functions but  */
/*  are made available to interested users and for completeness             */
/*                                                                          */
/*   GplGammaCreateTable ( PBYTE pbGamma,                                   */
/*                                 LONG  lRgamma,                           */
/*                                 LONG  lGgamma,                           */
/*                                 LONG  lBgamma,                           */
/*                                 LONG  lbias );                           */
/*                                                                          */
/* GplGammaColorTableToGamma2 simply for 8bit pels or less, gamma replaces  */
/* the RGB values in the PBITMAPINFO2 argbColor[0..256] structure.          */
/*                                                                          */
/*   GplGammaColorTableToGamma2 ( PBYTE pbGamma,                            */
/*                                   PBITMAPINFO2 pSrcInfo );               */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* Modification/Change log                                                  */
/* jk0998 - "Void and Cluster"  added                                       */
/*--------------------------------------------------------------------------*/
/* TAG - DATE - [COMPANY] - AUTHOR - DESCRIPTION                            */
/*--------------------------------------------------------------------------*/
/* jk0998 - September 1998                                                  */
/* Janis Kisis, SWH Technology, Riga, Latvia                                */
/****************************************************************************/

/* Different dithering algorithms to choose from
*/
#define HT_LEVEL                             0
#define HT_SNAP                              1
#define HT_DITHER_4x4                        2
#define HT_DITHER_8x8                        3
#define HT_ERR_DIFFUSION                     4
#define HT_MAGIC_SQUARES                     5
#define HT_ORDERED_SQUARES                   6
#define HT_ADAPTIVE_DIFFUSION                7
#define HT_FAST_DIFFUSION                    8
#define HT_STEINBERG_DIFFUSION               9
#define HT_SMOOTH_DIFFUSION                  10
#define HT_HSV_DIFFUSION                     11
#define HT_HSV_BIDIFFUSION                   12
#define HT_CMYK_DIFFUSION                    13
#define HT_VOID_CLUSTER                      14         // jk0998 - was DRAFT
#define HT_JANIS_STUCKI                      15         // jk0998 - was PHOTO
#define HT_ERR_DIFFUSION5                    0x54       // sx0999
#define HT_ERR_DIFFUSION6                    0x64       // sx0999

#define HT_PARAMETER                         0x00
#define HT_MIN_COLOR                         0x10
#define HT_PERCENT                           0x20
#define HT_FUNCTION                          0x30
#define HT_5_COLOR                           0x50       // sx0999
#define HT_6_COLOR                           0x60       // sx0999

/* Classes or categories of dithering groups
*/
#define DITHER_ID_MATRIX                     1
#define DITHER_ID_DIFFUSION                  2
#define DITHER_ID_HSV_DIFFUSION              3
#define DITHER_ID_CMYK_DIFFUSION             4
#define DITHER_ID_VOID_CLUSTER               5      // jk0998
#define DITHER_ID_NEW_DIFFUSION              6      // jk0998
#define DITHER_ID_NEW_MATRIX                 7      // sx

/* ---------------------from omni\device.h --------------------- */
// The following are defines for the different Haltoning Algorithms.
// These are values for JobProperties.usAlgorithm
//

#define COLOR_TECH_MONO                      0
#define COLOR_TECH_COLOR                     1
#define COLOR_TECH_K                         2
#define COLOR_TECH_CMY                       3
#define COLOR_TECH_CMYK                      4
#define COLOR_TECH_KCMY                      5

#define MODE_ID_BLK_COLOR                    1
#define MODE_ID_COLOR                        2

#define MEDIA_ID_PLAIN                       1
#define MEDIA_ID_1_SPECIAL                   2
#define MEDIA_ID_2_SPECIAL                   3
#define MEDIA_ID_GLOSSY                      4
#define MEDIA_ID_TRANS                       5

#define RES_ID_75                            1
#define RES_ID_90                            2
#define RES_ID_100                           3
#define RES_ID_120                           4
#define RES_ID_150                           5
#define RES_ID_180                           6
#define RES_ID_240                           7
#define RES_ID_300                           8
#define RES_ID_360                           9
#define RES_ID_400                           10
#define RES_ID_500                           11
#define RES_ID_600                           12
#define RES_ID_720                           13
#define RES_ID_800                           14
#define RES_ID_900                           15
#define RES_ID_1000                          16
#define RES_ID_1100                          17
#define RES_ID_1200                          18

#define REQUEST_DEFAULT_REDUCTION            -1L

//
//    bReq  -  color request type
//
#define CMYK_SIZE                            16
#define CMYK_CYAN                            0
#define CMYK_MAGENTA                         1
#define CMYK_YELLOW                          2
#define CMYK_BLACK                           3

#define CMY_ALL                              4
#define CMYK_ALL                             5


#define CMYK_FILTER_OVERFLOW                 8

#define BUMP_TO_NEXT_MODULUS(n,m)            ((((n)+(m)-1)/(m))*(m))

#define MAX3(a, b, c)                        (a > b ? (a > c ? a : c) : b > c ? b : c)

#define MIN3(a, b, c)                        (a < b ? (a < c ? a : c) : b < c ? b : c)

#define INVALID_PARAMETER                    INVALID_BITMAP

#define SET_VALUE_IN_ERR_BUFFS               0
#define COPY_1ST_ROW_TO_ERR_BUFFS            1
#define DON_T_TOUCH_ERR_BUFFS                2

    // Select appropriate dither

#define GPL_DITHER_SETUP_DEFAULTS(Req, ulLevel)                     \
      switch (Req.lDitherType)                                      \
      {                                                             \
      case HT_LEVEL:                                                \
      {                                                             \
         Req.snap.ulParm = ulLevel & 0xff;                          \
         break;                                                     \
      }                                                             \
      case HT_ERR_DIFFUSION:                                        \
      case HT_ERR_DIFFUSION6:                                       \
      case HT_ERR_DIFFUSION5:                                       \
      case HT_FAST_DIFFUSION:                                       \
      case HT_ADAPTIVE_DIFFUSION:                                   \
      case HT_STEINBERG_DIFFUSION:                                  \
      {                                                             \
         Req.SrcOptions.fResetErrBuf = TRUE;                        \
         Req.SrcOptions.ulResetErrBufs = SET_VALUE_IN_ERR_BUFFS;    \
         Req.SrcOptions.ulValue = 0;                                \
         break;                                                     \
      }                                                             \
      case HT_DITHER_4x4:                                           \
      case HT_DITHER_8x8:                                           \
      case HT_MAGIC_SQUARES:                                        \
      case HT_ORDERED_SQUARES:                                      \
      default:                                                      \
         break;                                                     \
      }

#define GPL_DITHER_RESET_ERR_BUFFERS(Req)                           \
      if (  Req.lDitherType == HT_ERR_DIFFUSION                     \
         || Req.lDitherType == HT_ERR_DIFFUSION6                    \
         || Req.lDitherType == HT_ERR_DIFFUSION5                    \
         || Req.lDitherType == HT_FAST_DIFFUSION                    \
         || Req.lDitherType == HT_STEINBERG_DIFFUSION               \
         || Req.lDitherType == HT_ADAPTIVE_DIFFUSION                \
         )                                                          \
         Req.SrcOptions.fResetErrBuf = TRUE;

#define GPL_DITHER_PLANES_BLANK(Reply)                              \
       (  (LONG)Reply.fEmpty.bBlack                                 \
       && (LONG)Reply.fEmpty.bCyan                                  \
       && (LONG)Reply.fEmpty.bMagenta                               \
       && (LONG)Reply.fEmpty.bYellow                                \
       && (LONG)Reply.fEmpty.bLightCyan                             \
       && (LONG)Reply.fEmpty.bLightMagenta                          \
       && (LONG)Reply.fEmpty.bLightYellow                           \
       )

//#define GPL_DITHER_PLANES_BLANK6(Reply6)                            \
//       (  (LONG)Reply6.fEmpty.bBlack                                \
//       && (LONG)Reply6.fEmpty.bCyan                                 \
//       && (LONG)Reply6.fEmpty.bMagenta                              \
//       && (LONG)Reply6.fEmpty.bLightCyan                            \
//       && (LONG)Reply6.fEmpty.bLightMagenta                         \
//       && (LONG)Reply6.fEmpty.bYellow                               \
//       )

/*-------------------------- typedefs ------------------------*/
/***************************************************************
pReq     DITHEREQUEST Structure Element Description :
****************************************************************

bReq           (BYTE)

         CMY_ALL        Used for 3 color rendering. Output buffers are
                        consecutive byte aligned on each row, and grouped
                        in CMY order.

         CMYK_ALL       Render 3 colors and black. Output buffers are
                        consecutive byte aligned on each row, and grouped
                        in KCMY order.

         --------------------------------------------------------------------

lDitherType    (LONG)

        HT_LEVEL        if 0 then snap.ulParm contains a value 1..254 that
                        separates 0 & 1
            HT_PERCENT     if OR'd 0x80 then snap.ulParm is a percentage 1..100 that
                           separates 0 & 1
            HT_FUNCTION    if OR'd 0x40 then snap.pFunction will be executed to get a
                           midpoint with the following parameters:
                                pSrc
                                pbmi
                                pdest
                                pdestInfo

        ........... MATRIX HALFTONE SCREENS ..........

        HT_DITHER_4x4   Use's a 4x4  halftone pattern for 16 levels of
                        percieved intensity.  Also "snap.pHalftone" must be a
                        pointer to a 16x4x4 byte array of dither patterns.

        HT_DITHER_8x8   Use's 8x8  halftone pattern for 64 levels of
                        percieved intensity.Also "snap.pHalftone" must be a
                        pointer to a 16x8x8 byte array of dither patterns.

        HT_MAGIC_SQUARES Use Magic Squares HSV transform for nearest
                        color and dithering.

        HT_ORDERED_SQUARES Uses Recursive Tesselated Tiles of Rectangular Threshold
                        Arrays; i.e. Ordered squares. Uses the same function as
                        HT_MAGIC_SQUARES but a different dither table/pattern.

        .......... DIFFUSION .........

        HT_ERR_DIFFUSION Use's Stucki Error Diffusion, Low Pass filter diffusion
                        This operates as a lag-filter and has a slow response
                        to sharp color changes, such as from white to any
                        dark color.

        HT_ERR_DIFFUSION5 and
        HT_ERR_DIFFUSION6 Use's Stucki Error Diffusion, Low Pass filter diffusion
                        This operates as a lag-filter and has a slow response
                        to sharp color changes, such as from white to any
                        dark color. I'll write later if it will be happened

        HT_STEINBERG_DIFFUSION Use's Steinberg Error Diffusion, Similar to Stucki
                        but has a alightly better response to sharp color
                        changes but slightly less overall quality.

        HT_FAST_DIFFUSION Uses Spurrier and Cooper Error Diffusion, High Pass filter
                        diffusion and works well with low gamma values and
                        sharp color changes but sometimes washed-out.

        HT_VOID_CLUSTER Use's  Robert Ulichney void-and-Cluster method for dither
            array generation  // jk0998


        OTHER Dither functions will be added from time to time.

         --------------------------------------------------------------------

SrcOptions     (SRCMODS) Structure

        fStartOfPage    (BOOL)
                        Set in users NewFrame function, Initialized by SYSTEM

        fNewColors      (BOOL)
                        Set to TRUE by caller when a new color table has been
                        selected.(If 8 bit pels or less). Reset to FALSE by this program.

                        e.g. :
                        pReq->SrcOptions.fNewColors        = TRUE;

        fModify         (BOOL)
                        TRUE if pSrc may be modified during conversion,
                        otherwise, if FALSE or zero, pels in pSrc will NOT
                        be modified.

        fResetErrBuf    (BOOL)
                        Set between pages or when Err Buffers must be reset

        ulResetErrBufs  (ULONG)
                        =0 reset ErrorDiffusion buffers to white (255)
                        =1 set ErrorDiffusion buffers to ..SrcOptions.iValue
                        =2 duplicate the corresponding values from row one as
                           initial values for the last row. i.e. start at one
                           row before the first row which looks like the first
                           row
                        =3 don't touch
        ulValue         (ULONG)
                        (0..256) << 20. A BYTE value shifted 20 places left.

         --------------------------------------------------------------------

snap           (SNAPIT)

                        This is effectively a LONG but has 4 possible representations
                        as described in the iDitherType above parameter.

                        union {
                           PBYTE       pFunction;
                           PRGB2       pHalftone;
                           ULONG       ulParm;
                           PBYTE       pArray;
                        }  SNAPIT;

         --------------------------------------------------------------------

ulResolution    (ULONG)        The larger of x or y resolution

         --------------------------------------------------------------------

iNumColors     (INT)

                        This parameter must contain one of the following:

                        2, 4, 8, 16, 256, 64k, 16meg

         --------------------------------------------------------------------

iSrcRowPels    (INT)
                        Number of Pels in a source row.

         --------------------------------------------------------------------

iNumDitherRows   (INT)
                        Number of rows in a band

         --------------------------------------------------------------------

pSrcInfo       (PBITMAPINFO2) Structure

                        BITMAPINFO2 defined in OS/2 PM Prog Ref Vol III Appendix A
                        Pointer to a BITMAPINFO2 Structure.  Used primarily for
                        cx, cy, and the argbColor 8 bit indexed 256 color table

         --------------------------------------------------------------------

pSrc            (PBYTE)

                        Pointer to source RGB2 bitmap which must be 32bit aligned
                        such that each row begins on a 32bit boundry.

                        NOTE:    The user must have preserved this pointer

         --------------------------------------------------------------------

iNumSrcRowBytes32  (INT)     iDWordBytesInCMYKBandRow

          --------------------------------------------------------------------

iNumDestRowBytes   (INT)     iBytesInSinglePlaneBandRow

         --------------------------------------------------------------------

delta          (DELTACOLOR) Structure

                        This parameter NEED ONLY BE initialized for only Magic
                        and Ordered Squares.

        lHue            (LONG)
                        Number of degrees to rotate increase/decrease all hue
                        values. -255E8 < lHue < 255E8
                        ( 255 * DialogHue * E8 /360 )

        lSaturation     (LONG)
                        Percentage increase/decrease of purity of lHue.
                        i.e. the absence of White is when Saturation = 100% or
                        pure Color. Conversely when there is an absence of a
                        hue, Saturation = 0% or White when black is absent;
                        i.e. Value is 100%.  -255 <= lSaturation <= 255
                        ( 255 * DialogSaturation/100 )

        lValue          (LONG)
                        Percentage increase\decrease of black. Value is the
                        absence of black. If not absent of any black, its
                        Value is 0%.
                        -255 <= lValue <= 255. ( 255 * DialogValue/100 )

                        e.g. :
                        pReq->delta.iValue  = pddc->pdb->pJobProperties->lValue;
                        pReq->delta.lSaturation =
                              (255 * pReq->delta.iSaturation)  / 100;
                        pReq->delta.lValue = (255 * pReq->delta.lValue) / 100;

         --------------------------------------------------------------------

fRemoveBlackColor     (BOOL)
                        Eliminate coincident Black and Color after Dithering

         --------------------------------------------------------------------

lRGamma     (ULONG)    Red   Gamma  1 to 255 ( actually 10 times true gamma )
lRBias                 Red   Bias usually 0
lGGamma                Green Gamma  1 to 255 ( actually 10 times true gamma )
lGBias                 Green Bias usually 0
lBGamma                Blue  Gamma  1 to 255 ( actually 10 times true gamma )
lBBias                 Blue  Bias usually 0

                        Gamma is increased to reduce the color density, or
                        to lighten the picture.  When Gamma is increased, the
                        system generates a corresponding Bias.  Users
                        via the Printer Color Dialog, have the option to increase
                        or decrease whatever the system has determined is the
                        general estimated settings for the particular printer and
                        print configuration.

                        These values are actually 10 times the actual value used
                        A value of 10 therefore yields a gamma of 1 and therefore
                        the gamma function will have no effect: x(exp(1/gamma))=x

                        Any non-zero values supplied by these parameters will
                        override the system estimated baseline values, and user
                        modifications via the Dialog will increase or decrease
                        these values.

         --------------------------------------------------------------------
***************************************************************
pReply     DITHERESULT Structure Element Description :
***************************************************************

fEmpty         (CMYK) Structure

        bCyan           Initialized to TRUE on entry and stays TRUE when pSrc
        bMagenta        has been set to all white. Set to FALSE if more pel
        bYellow         processing is required. If "SrcOptions.fModify" is TRUE,
        bBlack          fEmpty can only be set to FALSE if all "pSrc" pels
                        are black and/or white.

         --------------------------------------------------------------------

pbKBuffer      (PBYTE)    ptr to 8bit row aligned output bitmap
pbXBuffer      (PBYTE)    ptr to 8bit row aligned output bitmap
pbYBuffer      (PBYTE)    ptr to 8bit row aligned output bitmap
pbZBuffer      (PBYTE)    ptr to 8bit row aligned output bitmap

---------------------End of pReq--------------------

*/
/*------------------------ Request structures ---------------------------*/
//
//    User color modifications for SQUARES algorithm from JobProperties
//
typedef struct _DELTACOLORstruct {
   LONG        lHue;
   LONG        lSaturation;
   LONG        lValue;
} DELTACOLOR, *PDELTACOLOR;

//
//    Midpoint and function control parameters structure
//
typedef union {                    // PRGB2 defined in OS/2 PM Prog Ref Vol III Appendix A
   PRGB2       pHalftone;          // Pointer to 4x4 8x8 or user supplied table
   ULONG       ulParm;             // SYSTEM initialized or user supplied midpoint or function parameter
   PBYTE       pArray;             // Pointer to 16x16 tables such as paintMixer, OrderSquares, or usersupplied
   PBYTE       pFunction;          // Reserved for User supplied functions TBD
} SNAPIT, *PSNAPIT;

//
//    Flags that indicate the state of the buffer information structure
//
typedef struct _SRCMODSstruct {
   BOOL        fStartOfPage;       // Set in users NewFrame function, Initialized by SYSTEM
   BOOL        fNewColors;         // Set if pSrcInfo Color Table changes, Initialized by SYSTEM
   BOOL        fModify;            // Set if pSrc may be modified for K (Black) when CMY set
   //
   // Set by MACRO's GPL_DITHER_SETUP_DEFAULTS, GPL_DITHER_RESET_ERR_BUFFERS and/or user
   //
   BOOL        fResetErrBuf;       // Set by MACRO between pages or when Err Buffers must be reset
   ULONG       ulResetErrBufs;     // 0,1,2,or 3
   ULONG       ulValue;            // xxx << 20 value to be set as initialization value
} SRCMODS, *PSRCMODS;

//
//   REQUEST Structure to be set up by the User
//
typedef struct _DITHERREQUESTstruct {
   BYTE          bReq;              // Set to CMY_ALL or CMYK_ALL
   LONG          lDitherType;       // From JobProperties
   SRCMODS       SrcOptions;        // Flags that indicate the state of the buffer information
   SNAPIT        snap;              // Midpoint and function control parameters
   //
   // Source bitmap configuration information
   //
   ULONG         ulResolution;      // The larger of x or y resolution
   INT           iNumColors;
   INT           iSrcRowPels;       // Number of Pels in a source row
   INT           iNumDitherRows;    // Number of rows in a band
   PBITMAPINFO2  pSrcInfo;          // if new colors, deallocate old
                                    // BITMAPINFO2 defined in OS/2 PM Prog Ref Vol III Appendix A
   PBYTE         pbSrc;             // ptr to input bitmap (rows are 32bit aligned)
   INT           iNumSrcRowBytes32; // iDWordBytesInCMYKBandRow;
   INT           iNumDestRowBytes;  // iBytesInSinglePlaneBandRow;
   DELTACOLOR    delta;             // From JobProperties
   //
   // Destination bitmap configuration information
   //
   INT           iDestBitsPerPel;   // How many bits
   //
   // Color adjustment parameters
   //
   BOOL          fRemoveBlackColor; // Eliminate coincident Black and Color after Dithering
   LONG          lRGamma;
   ULONG         ulRBias;           //  0 .. 255 gamma table minimum value
   LONG          lGGamma;
   ULONG         ulGBias;           //  0 .. 255 gamma table minimum value
   LONG          lBGamma;
   ULONG         ulBBias;           //  0 .. 255 gamma table minimum value
   LONG          lKGamma;
   ULONG         ulKBias;           //  0 .. 255 gamma table minimum value
   LONG          lBlackReduction;   //  0 .. 100 % black reduction
   LONG          lCGamma;
   LONG          lMGamma;
   LONG          lYGamma;
   float        fLightCoefficient;
} DITHEREQUEST, *PDITHEREQUEST;

/*------------------------ Results structures ---------------------------*/

typedef struct _EMPTYstruct {
   BYTE        bCyan;
   BYTE        bMagenta;
   BYTE        bYellow;
   BYTE        bBlack;
   BYTE        bLightCyan;
   BYTE        bLightMagenta;
   BYTE        bLightYellow;
} CMYK, *PCMYK;

//typedef struct _EMPTYstruct6 {
//   BYTE        bCyan;
//   BYTE        bMagenta;
//   BYTE        bYellow;
//   BYTE        bBlack;
//   BYTE        bLightCyan;
//   BYTE        bLightMagenta;
//   BYTE        bLightYellow;
//} CMYK6, *PCMYK6;

typedef struct _DITHERESULTstruct {
   CMYK          fEmpty;            // TRUE when no pels of a color on the row are set for printing or black
   PBYTE         pbKBuffer;         //  ptr to 8bit row aligned output bitmap
   PBYTE         pbXBuffer;         //  ptr to 8bit row aligned output bitmap
   PBYTE         pbYBuffer;         //  ptr to 8bit row aligned output bitmap
   PBYTE         pbZBuffer;         //  ptr to 8bit row aligned output bitmap
   PBYTE         pbXLBuffer;        //  ptr to 8bit row aligned output bitmap
   PBYTE         pbYLBuffer;        //  ptr to 8bit row aligned output bitmap
   PBYTE         pbZLBuffer;        //  ptr to 8bit row aligned output bitmap
} DITHERESULT, *PDITHERESULT;

//typedef struct _DITHERESULTstruct6 {
//   CMYK6         fEmpty;             // TRUE when no pels of a color on the row are set for printing or black
//   PBYTE         pbKBuffer;          //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbXBuffer;          //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbYBuffer;          //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbZBuffer;          //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbXLBuffer;         //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbYLBuffer;         //  ptr to 8bit row aligned output bitmap
//   PBYTE         pbZLBuffer;         //  ptr to 8bit row aligned output bitmap
//} DITHERESULT6, *PDITHERESULT6;

#define DITHER_GAMMA_RED                 1
#define DITHER_GAMMA_GREEN               2
#define DITHER_GAMMA_BLUE                3
#define DITHER_GAMMA_BLACK               4

/*------------------------ API prototypes ---------------------------*/

APIRET APIENTRY GplDitherCreateInstance    (PPVOID           ppdh,
                                            HMCB             hmcb,
                                            PDITHEREQUEST    pReq,
                                            PDITHERESULT     pReply);

APIRET APIENTRY GplDitherRGBtoCMYK         (PPVOID           ppdh,
                                            PDITHEREQUEST    pReq,
                                            PDITHERESULT     pReply);

APIRET APIENTRY GplDitherRGBtoRGB          (PPVOID           ppdh,
                                            PDITHEREQUEST    pReq,
                                            PDITHERESULT    pReply);

APIRET APIENTRY GplDitherDeleteInstance    (PVOID            pDH);

APIRET APIENTRY GplGammaCreateTable        (PBYTE            pbGamma,
                                            LONG             lRgamma,
                                            LONG             lGgamma,
                                            LONG             lBgamma,
                                            LONG             lKgamma,
                                            LONG             lbias);

APIRET APIENTRY GplGammaColorTableToGamma2 (PBYTE            pbGamma,
                                            PBITMAPINFO2     pbmi);

APIRET APIENTRY GplGammaConvertRGBtoCMY    (LONG             lRgamma,
                                            LONG             lGgamma,
                                            LONG             lBgamma,
                                            PLONG            plCGamma,
                                            PLONG            plMGamma,
                                            PLONG            plYGamma);

PBYTE APIENTRY  GplDitherQueryGamma        (PPVOID           ppdh,
                                            ULONG            ulId);

/*----------------------------------------------------
                 EXAMPLE SETUPS
----------------------------------------------------

------------------- End of Examples ------------------
*/

#endif
