/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = DLGPAGE.H
 *
 * DESCRIPTION = Public output dialog header for GENPLIB.LIB
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 04/04/96
 * AUTHOR  = Larry Moore
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_dlgpage_h
#define _gpl_dlgpage_h

/*****************************************************************************\
** DEFINES                                                                   **
\*****************************************************************************/
/*
** Default string size for the GENPLIB string arrays.
*/
#define GDP_STRING_SIZE       64

/*
** Arguments for sStrType in GplPageInsertListItems().  GDP_STRTYPEPTR means
** that the list of values are string addresses.  GDP_STRTYPEID means that
** the list of values are resource IDs.
*/
#define GDPF_STRTYPEPTR        1
#define GDPF_STRTYPEID         2

/*
** Used in GPTMPL.ulDlgID.  If set, then the page is not inserted.
*/
#define GDPF_DO_NOT_INSERT     0

/*
** Used in GplPageSelectListItem().  GDPI_HANDLE means that the value passed
** in is a listbox item handle and to select the item based on that handle.
** GDPI_INDEX means to select a item, given an index value.
*/
#define GDPI_HANDLE       1
#define GDPI_INDEX        2

#ifndef BKM_QUERYPAGEINFO
  #define BKM_QUERYPAGEINFO        0x0368
#endif





/*****************************************************************************\
** TYPEDEFS                                                                  **
\*****************************************************************************/
typedef ULONG  GDPDH;
typedef ULONG  STRID, *PSTRID;

/******************************************************************************
** GENPLIB Page Template
** This structure makes up a data of dialog notebook page templates that
** are to be inserted into a notebook using GplPageInsert().
** The order of templates in a GDPTMPL array determines how the pages will
** be order in the notebook.
**
** LIMITIATIONS:
** - ulDlgID must be greater than 0.
** - ulTabString may either be a pointer, or a resource ID.  If a resource
**   ID is used, ulFlag must enable GTAB_ID.
**
** If it is desired not to create a page from a specific template, then
** set ulDlgID to GDPF_DO_NOT_INSERT.  In that case, the function will
** ignore the current template.
******************************************************************************/
typedef struct _GDPTMPL
{
  ULONG ulDlgID;                /* Dialog ID - cannot be 0                   */
  PFNWP pfnDlgProc;             /* Window procedure of above dialog ID       */
  ULONG ulTabString;            /* ID or pointer to tab string               */
  ULONG ulFlag;                 /* General flags                             */
        #define GTAB_ID    1
  ULONG ulReserved;             /* Reserved - must be 0                      */
} GDPTMPL, *PGDPTMPL;


/******************************************************************************
** GENPLIB Page Information
** This structure contains specific information about a page.
** - ulPageID is the ID given by the notebook.  If you need to access specific
**   pages using notebook message, use this field.
** - ulDlgID is the ID given by GDPTMPL.ulDlgID.
** - ulArg1 and ulArg2 are reserved for the user to modify.
******************************************************************************/
typedef struct _GDPPI
{
  HWND  hPage;                       /* Notebook page handle                 */
  ULONG ulPageID;                    /* Notebook page ID                     */
  ULONG ulDlgID;                     /* Dialog ID provided from template     */
  ULONG ulArg1;                      /* Application-specific data            */
  ULONG ulArg2;                      /* Application-specific data            */
} GDPPI, *PGDPPI;





/*****************************************************************************\
** FUNCTION PROTOTPYES                                                       **
\*****************************************************************************/
extern BOOL APIENTRY GplPageSetDlgParams( HWND, ULONG, ULONG, PSZ );
extern LONG APIENTRY GplPageInsert( HWND, ULONG, PGDPTMPL, USHORT, HMODULE,
                                    PVOID, PGDPPI );
extern LONG APIENTRY GplPageRemove( HWND );
extern LONG APIENTRY GplPageInsertListItems( HWND, ULONG, PSTRID, LONG,
                                             PLONG, ULONG, SHORT, HMODULE );
extern BOOL APIENTRY GplPageSelectListItem( HWND, ULONG, LONG, LONG );
extern BOOL APIENTRY GplPageQueryLBSelection( HWND, LONG, PLONG, PSHORT );

#endif
