/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
#ifndef FONTMGR_H

#define FONTMGR_H

// versions for Font Manager preferences and Font profile
#define FNTMGR_VERSION                  0x0100
#define FNTMGR_FONT_VERSION             0x0100

// storage type
#define STORAGE_TYPE_UNKNOWN            0
#define STORAGE_TYPE_HARDDISK           1
#define STORAGE_TYPE_FLASH              2
#define STORAGE_TYPE_RAM                3
#define STORAGE_TYPE_DEFAULT            STORAGE_TYPE_RAM

// storage type alias
#define STORAGE_TYPE_UNKNOWN_ALIAS      "Unknown"
#define STORAGE_TYPE_HARDDISK_ALIAS     "Harddisk"
#define STORAGE_TYPE_FLASH_ALIAS        "Flash"
#define STORAGE_TYPE_RAM_ALIAS          "RAM"

// @V1.LAST
//
// key aliases for Font Profile keys
#define FONT_PROFILE_PREFIX             "FNTMGR_FONTID_"
#define FONT_PROFILE_BINARYPATH         "BINARYPATH"
#define FONT_PROFILE_PRINTEROUTPUT      "PRINTEROUTPUT"
#define FONT_PROFILE_FONTNAME           "NAME"
#define FONT_PROFILE_DEVICENAME         "DEVICENAME"
#define FONT_PROFILE_FONTTYPE           "TYPE"
#define FONT_PROFILE_FONTSTORAGE        "STORAGE"

#define MAXFONTNAME                     CCHMAXPATH

// font binaries type
#define FONT_TYPE_TYPE1                 "TYPE1"
#define FONT_TYPE_TRUETYPE              "TRUETYPE"
#define FONT_TYPE_TRUETYPEXL            "TRUETYPEXL"
#define FONT_TYPE_INTELLIFONT           "INTELLIFONT"

// @V1.LAST
//
// assign index values to Font profile keys
#define FONT_KEY_INDEX_START            0
#define FONT_KEY_INDEX_BINARYPATH       (FONT_KEY_INDEX_START + 0)
#define FONT_KEY_INDEX_PRINTEROUTPUT    (FONT_KEY_INDEX_START + 1)
#define FONT_KEY_INDEX_FONTNAME         (FONT_KEY_INDEX_START + 2)
#define FONT_KEY_INDEX_DEVICENAME       (FONT_KEY_INDEX_START + 3)
#define FONT_KEY_INDEX_FONTTYPE         (FONT_KEY_INDEX_START + 4)
#define FONT_KEY_INDEX_FONTSTORAGE      (FONT_KEY_INDEX_START + 5)
#define FONT_KEY_INDEX_NULL             (FONT_KEY_INDEX_START + 6)
#define FONT_KEY_INDEX_TOTAL            7

// @V1.LAST
//
// general information about dynamic font
typedef struct _FONT_PROFILE_1X   FONT_PROFILE_1X, *PFONT_PROFILE_1X;
struct _FONT_PROFILE_1X
{
   char   szFontBinary[CCHMAXPATH];      // font binary pathname
   char   szPrinterOutput[CCHMAXPATH];   // printer output pathname
   char   szFontName[MAXFONTNAME];       // OS font name
   char   szDeviceName[MAXFONTNAME];     // device font name
   char   szFontType[16];                // font type
   BYTE   bStorageType;                  // location
};

// general information about dynamic font
typedef struct _FONT_PROFILE   FONT_PROFILE, *PFONT_PROFILE;
struct _FONT_PROFILE
{
   ULONG    ulSize;                        // profile size
   USHORT   usVersion;                     // profile version
   char     szFontBinary[CCHMAXPATH];      // font binary pathname
   char     szPrinterOutput[CCHMAXPATH];   // printer output pathname
   char     szFontName[MAXFONTNAME];       // OS font name
   char     szDeviceName[MAXFONTNAME];     // device font name
   char     szFontType[16];                // font type
   BYTE     bStorageType;                  // location
   SHORT    FontID;                        // used in PCL5
};

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILERECOGNIZE)(
   PFONT_PROFILE   prFontProfile
   );

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILEDOWNLOAD)(
   PSZ             pszQueueName,
   PFONT_PROFILE   prFontProfile
   );

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILEUNINSTALL)(
   PSZ             pszQueueName,
   PFONT_PROFILE   prFontProfile
   );

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILERELOAD)(
   PSZ             pszQueueName,
   PFONT_PROFILE   prFontProfile
   );

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILECREATE)(
   PFONT_PROFILE   prFontProfile
   );

//
typedef BOOL
(APIENTRY _FNFONTFUNC_FILEDELETE)(
   PFONT_PROFILE   prFontProfile
   );

typedef _FNFONTFUNC_FILERECOGNIZE   *PFNFONTFUNC_FILERECOGNIZE;
typedef _FNFONTFUNC_FILEDOWNLOAD    *PFNFONTFUNC_FILEDOWNLOAD;
typedef _FNFONTFUNC_FILEUNINSTALL   *PFNFONTFUNC_FILEUNINSTALL;
typedef _FNFONTFUNC_FILERELOAD      *PFNFONTFUNC_FILERELOAD;
typedef _FNFONTFUNC_FILECREATE      *PFNFONTFUNC_FILECREATE;
typedef _FNFONTFUNC_FILEDELETE      *PFNFONTFUNC_FILEDELETE;

// dispatch table for FONTFUNC manager
typedef struct _FONTFUNC_DISPATCH   FONTFUNC_DISPATCH, *PFONTFUNC_DISPATCH;
struct _FONTFUNC_DISPATCH
{
   PFNFONTFUNC_FILERECOGNIZE   pfnFontFuncFileRecognize;
   PFNFONTFUNC_FILEDOWNLOAD    pfnFontFuncFileDownload;
   PFNFONTFUNC_FILEUNINSTALL   pfnFontFuncFileUninstall;
   PFNFONTFUNC_FILERELOAD      pfnFontFuncFileReload;
   PFNFONTFUNC_FILECREATE      pfnFontFuncFileCreate;
   PFNFONTFUNC_FILEDELETE      pfnFontFuncFileDelete;
};

// dispatch functions shortcuts
#define FontFuncFileRecognize   rFontFuncDispatch.pfnFontFuncFileRecognize
#define FontFuncFileDownload    rFontFuncDispatch.pfnFontFuncFileDownload
#define FontFuncFileUninstall   rFontFuncDispatch.pfnFontFuncFileUninstall
#define FontFuncFileReload      rFontFuncDispatch.pfnFontFuncFileReload
#define FontFuncFileCreate      rFontFuncDispatch.pfnFontFuncFileCreate
#define FontFuncFileDelete      rFontFuncDispatch.pfnFontFuncFileDelete

// storage status
#define STORAGE_STATUS_ERROR     -1
#define STORAGE_STATUS_UNKNOWN   0

// general information about device which is under control of Font Manager
typedef struct _FNTMGR_DEVINFO   FNTMGR_DEVINFO, *PFNTMGR_DEVINFO;
struct _FNTMGR_DEVINFO
{
   LONG   lHarddisk;   // printer hard disk capacity
   LONG   lFlash;      // printer flash RAM capacity
   LONG   lRAM;        // printer RAM amounts
};

// general information structure - passed to Font Manager during initialization
typedef struct _FNTMGR_DRIVERINFO   FNTMGR_DRIVERINFO, *PFNTMGR_DRIVERINFO;
struct _FNTMGR_DRIVERINFO
{
   PSZ                 pszPrinterName;      // printer profile key
   FONTFUNC_DISPATCH   rFontFuncDispatch;   // supported font operations
   FNTMGR_DEVINFO      rDevInfo;            // supported locations
};

//
typedef struct _GFMCB
{
   ULONG   ulReserved;  // Reserved - must be 0
} GFMCB, *PGFMCB, **PPGFMCB;

// GENPLIB Font Manager Callback Procedure
typedef MRESULT (APIENTRY GFMCP) (PGFMCB, PVOID);
typedef GFMCP   *PGFMCP, **PPGFMCP;

//
HGJPI APIENTRY
GplFontMgrCreatePage(
   HGJNI                hNBInstance,
   PGFMCB               pFMCB,
   PGFMCP               pCallbackProc,
   PVOID                pInstanceData,
   PFNTMGR_DRIVERINFO   prFntMgrDriverInfo
   );

//
LONG APIENTRY
GplFontMgrDestroyPage(
   HWND    hPage,
   HGJPI   hPageInstance
   );

//
LONG APIENTRY
GplFontMgrInitCB(
   PGFMCB   pFMCB
   );

//
LONG APIENTRY
GplFontMgrRefresh(
   HGJPI                hPageInstance,
   PFNTMGR_DRIVERINFO   prFntMgrDriverInfo
   );

// call this function in driver to be sure that Font Manager is supported
BOOL APIENTRY
GplFontMgrCheckEnvironment(
   VOID
   );

#define CHARSET_SIZE   256

//
typedef struct _DYNFNT_CHARSET DYNFNT_CHARSET, *PPDYNFNT_CHARSET;
struct _DYNFNT_CHARSET
{
   USHORT   usCodePage;
   USHORT   usCharSet[CHARSET_SIZE];
   BYTE     bmCharInUse[CHARSET_SIZE/8];
   USHORT   usCharUsage[CHARSET_SIZE];
};
   
// general structure for list of fonts which are donwloaded to the printer by
// Font Manager
typedef struct _DYNFNT_LIST   DYNFNT_LIST, *PDYNFNT_LIST;
struct _DYNFNT_LIST
{
   BOOL             fUsed;             // TRUE when font is realized
   ULONG            ulRefCount;        // References count
   ULONG            ulDeviceFont;      // Device font handle
   int              FontID;            // assigned index
   FONT_PROFILE     rFontProfile;      // general font information
   
   FATTRS           rFontAttributes;   // copy of RelizeFont parameters
   DYNFNT_CHARSET   CharSet;           // character set
   FONTMETRICS      FontMetrics;       // OS2 Font Metrics
   PFOCAFONT        pffGreFont;        // Gre Font Handle

   PDYNFNT_LIST     prNext;            // next font
};
   
// call this function to add font profile into the list
PDYNFNT_LIST
DynFntListAddItem(
   PDYNFNT_LIST*   pprDynFntListHead,
   PFONT_PROFILE   prFontProfile
   );
   
// supported search criteria
#define DYNFNT_SEARCH_FONTNAME     1L
#define DYNFNT_SEARCH_DEVICENAME   2L
#define DYNFNT_SEARCH_GREFONT      4L
#define DYNFNT_SEARCH_DEVICEFONT   8L
#define DYNFNT_SEARCH_FONTID       16L

// call this function with criteria to find the desired font in the list
PDYNFNT_LIST APIENTRY
DynFntListSearchItem(
   PDYNFNT_LIST   prDynFntListHead,
   ULONG          ulSearchItem,
   ULONG          ulItem
   );
   
// call this function to delete font from a list
VOID
DynFntListDeleteItem(
   PDYNFNT_LIST*   pprDynFntListHead,
   PSZ             pszFontName
   );

// release all allocated resources by font list
VOID APIENTRY
DynFntListDone(
   PDYNFNT_LIST   prDynFntListHead
   );

// this is prototype to all functions, that may be supplied with DynFntListInit
// call
typedef BOOL (* APIENTRY PFNLISTITEM_FUNC)(
   PFONT_PROFILE   prFontProfile,
   PVOID           pData
   );
   
// call this function to build font list downloaded to the specified printer by
// Font Manager
PDYNFNT_LIST APIENTRY
DynFntListInit(
   PSZ                pszPrinterName,
   PFNLISTITEM_FUNC   pfnListItemFunc,
   PVOID              pData
   );

// call this function to get list items count
USHORT APIENTRY
DynFntListQueryItemsCount(
   PDYNFNT_LIST   prDynFntListHead
   );
   
// call this function to get list item by specified index
PDYNFNT_LIST APIENTRY
DynFntListQueryItem(
   PDYNFNT_LIST   prDynFntListHead,
   USHORT         usItemIndex
   );

// DynFntCharSet works with DynFnt Character Set
#define DYNFNT_CS_SET_CP    1L
#define DYNFNT_CS_GET_CP    2L

//
ULONG APIENTRY
DynFntCharSet(
   PDYNFNT_LIST   prDynFnt,
   ULONG          ulOperation,
   ULONG          ulParam
   );

// call this function to know font must be simulated or not
BOOL APIENTRY
DynFntCheckAttributes(
   PDYNFNT_LIST   prDynFnt,
   USHORT         fFlags
   );
   
#endif
