/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = GLAYOUT.H
 *
 * DESCRIPTIVE NAME = Public header data for the Layout format.
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 08/21/97
 * AUTHOR  = Larry Moore
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_glayout_h
#define _gpl_glayout_h


/*
** This is a list of all sub-control ID's supported by the Layout control.
*/
#define GL_SET_DEFAULT_LAYOUT      GL_LAYOUT_NUP

// If set, then hide "Print page border".
#define GNDF_HIDEBORDERSELECTION   1

// we added this definitions from glayout.ph to solve DDK problem 
// for omni driver. The same was done for nup.h and num.ph for pscript.
/*****************************************************************************\
** DEFINES                                                                   **
\*****************************************************************************/
#define GDLID_BASE                  0x8200

#define GDLID_LAYOUTDIALOG          GDLID_BASE
#define GDLID_SAMPLEPAGE            (GDLID_BASE + 1)
#define GDLID_UNDO                  (GDLID_BASE + 2)
#define GDLID_DEFAULTS              (GDLID_BASE + 3)
#define GDLID_CONTEXTHELP           (GDLID_BASE + 4)




/*
** This union contains all of the possible "sub-control blocks" for Layout.
** Only one sub-control can be selected at a time.
*/
typedef union _GLOUT
{
  GJNCB   nUP;       // N-UP control block
  GJFTPCB uFTP;      // Fit To Page control block
  GJSCB   uScale;    // Scaling control block
} GLOUT, *PGLOUT;


/*
** Control block for Layout.  This contains all sub-controls.
*/
typedef struct _GLCB
{
  GJFC  gjfc;

  ULONG ulLayoutType;           // Current layout selected
        // N-UP
        #define GL_LAYOUT_NUP         0
        // Fit to page
        // LMMOORE
        #define GL_LAYOUT_FITTOPAGE   1
        // Scaling
        // BRIANM
        #define GL_LAYOUT_SCALE       2

  GLOUT layout;                 // List of sub-controls
  ULONG ulReserved;             // Reserved - must be 0
} GLCB, *PGLCB, **PPGLCB;

// GENPLIB Job Features Layout Callback Procedure
typedef MRESULT (APIENTRY GJLCP) (PGLCB, PVOID);
typedef GJLCP   *PGJLCP, **PPGJLCP;





LONG  APIENTRY GplLayoutVerifyCB( PGLCB );
LONG  APIENTRY GplLayoutInitCB( PGLCB, ULONG, ULONG );
HWND  APIENTRY GplLayoutCreatePage( HGJNI, PGLCB, PGJLCP, PVOID, PSZ *, ULONG,
                                    ULONG );
LONG  APIENTRY GplLayoutDestroyPage( HWND, HGJPI );
LONG  APIENTRY GplLayoutCopyCB( PGLCB, PGLCB );
LONG  APIENTRY GplLayoutSaveSettings( HGJPI );
HGJOI APIENTRY GplLayoutCreateInstance( HDC, PGLCB, PVOID );
LONG  APIENTRY GplLayoutDeleteInstance( HGJOI );
LONG  APIENTRY GplLayoutQuerySelected( HGJOI );
PVOID APIENTRY GplLayoutQueryControlBlock( HGJOI );
LONG  APIENTRY GplLayoutQueryParameters( HGJOI, PULONG, ULONG, ULONG );
LONG APIENTRY GplScalingQueryParameters( PGJSCB, PULONG, ULONG, ULONG );
BOOL  APIENTRY GplLayoutWriteProfile( PGLCB, PSZ );
BOOL  APIENTRY GplLayoutReadProfile( PGLCB, PSZ );
LONG  APIENTRY GplLayoutQueryCB( PVOID, LONG, PPGLCB );

#endif
