/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
#ifndef gpledc_h_
  #define gpledc_h_
//#############################################################################
//
// Public header for GplEDC - GeneralPublicLibaray EngineDrawnCharacters
//
//#############################################################################
// Note -  This currently does not support DBCS code pages
// See end of file for more tips
//

// Handy define for Euro char
#define EURO 213

#define FAIL -1
#define COM_SAVEHACK             0x10000000L

#ifndef PRIM_CHAR
#define PRIM_CHAR                2L
#endif
#define PRIM_CHAR_DONE           (PRIM_CHAR | 0x00000010)

// Pointer to function define
typedef LONG ( * _System PFUN )( );

// If you have trouble assigning an entry point try the following cast
// (LONG(*)())YourFunctionName
#define  _ENTRYCAST_ (LONG(*)())

// List of GRE/DRV entry points the package needs to work
typedef enum _ENTRYPOINTS { GREQUERYCHARPOSITIONS,
                            GRECHARSTRINGPOS,
                            HOOKEDFUNCTS } ENTRYPOINTS;

// Define for entry point set
typedef struct _GREDRVHOOKPAIR
{
  PFUN pGre;
  PFUN pDrv;
}
GREDRVHOOKPAIR, *PGREDRVHOOKPAIR;

// This is how the package saves entry points
typedef struct _GREDRVHOOKSET
{
  GREDRVHOOKPAIR HookPair[ HOOKEDFUNCTS ];
}
GREDRVHOOKSET, *PGREDRVHOOKSET;

// ABC space for char widths
typedef struct _ABCSPACE
{
   LONG     lA;
   ULONG    ulB;
   LONG     lC;
} ABCSPACE, *PABCSPACE;

// Info for a glyph
typedef struct _GLYPHDATA
{
  LONG      lUGL;
  LONG      lCharCode;  // Character codepoint for glyph in codepage
  ABCSPACE  ABCspace;
} GLYPHDATA, * PGLYPHDATA;

// Flags for GLYPHDATA.lCharCode
#define GLYPH_NOT_ACTIVE -1

#define CHARS_IN_CODEPAGE 256

// This struc holds the info needed to map one backup font.
// The driver needs to allocate on per backup font.
typedef struct _EDCFONTDATA
{
  ULONG ulFontHandle;       // Handle to recognize font, recommend value
                            // returned from GreRealizeFont
  ULONG ulFlags;            // Options, flags
  SHORT sCodePage;          // Active codepage glyphs are mapped to
  SHORT sGlyphCount;        // Count of glyphs
  LONG  lReMapTable[CHARS_IN_CODEPAGE];   // This table indicates which chars
                            // in glyphlist are actve for this codepage
  GLYPHDATA Glyph[1];       // An array of glyph info
} EDCFONTDATA, *PEDCFONTDATA;

// Flags for EDCFONTDATA.ulFlags
#define EDC_AUTOREMAP  0x00000001   // check CP on calls and remap if needed
#define EDC_SCALECHAR  0x00000002   // ABC space is scaled not raw
#define EDC_ACTIVE     0x00000004   // Mapping is active
#define EDC_NEED_REMAP 0x00000008   // Need A remap

// Flags for EDCFONTDATA.sCodePage
#define CP_NOT_DEFINED -1

// This define will get the amount of space in bytes need for the EDC font
// data.  To use:  where n is how many glyphs yiu want to remap
// p = SomeMemAllocFunc( EDCFONTDATA_SIZE( n ) );

#define EDCFONTDATA_SIZE(n) (sizeof(EDCFONTDATA)+(sizeof(GLYPHDATA) * ((n)-1)))

typedef struct _EDCGENDATA
{
  GREDRVHOOKSET HookTable;      // The hook Table
  PEDCFONTDATA  pEDCFontData;   // Ptr to Font data
  ULONG         ulDCFlags;      // Per DC flags
} EDCGENDATA, *PEDCGENDATA;

// Per DC flags
#define EDC_DC_NOT_ACTIVE  0x00000001  // Use this to override the fontdata
#define EDC_DC_IN_GPLCSP   0x00000002  // Indicates have entered the GPLCharStringPos

//*****************************************************************************
//
// FUNCTION: GplEDCSetUp
//
// This call is required.  It allows the driver to provide the entry points
// to be called later.  See ENTRYPOINTS list for what is needed.  A good place
// is in enable - fill physical device block when the driver has access to
// engine hook vector.
// For now the driver needs to know driver and gre entry points for
// GreQueryCharPositions and GreCharStringPos
//
// The first parm is hookset filled in with the entry points for CharStringPos
// and QueryCharPositions.
//
// The second parm is either a ptr to a EDCGENDATA struct or null.
// If your driver does not dynamically change the entry points you can use a 0
// meaning the EDC package will copy the hooktable to the per process space.
// If your driver does change the hooking then there should be a hook table
// for each DC.  Allocate sizeof(EDCGENDATA) and put it in or point to it with
// your Physical Device Block (pdv).  Pass the address of this block as the
// second parm.
//
// Example form FillPhysicalDeviceBlock:
// {
//   GREDRVHOOKSET  HookSet;
//   HookSet.HookPair[ GRECHARSTRINGPOS ].pGre =
//                   (PFUN)(od_memory_vect[ NGreCharStringPos & 0x00FF ].pfnl);
//   HookSet.HookPair[ GRECHARSTRINGPOS ].pDrv =
//                          (PFUN)(apfnDispatch[ NGreCharStringPos & 0x00FF ]);
//   HookSet.HookPair[ GREQUERYCHARPOSITIONS ].pGre =
//              (PFUN)(od_memory_vect[ NGreQueryCharPositions & 0x00FF ].pfnl);
//   HookSet.HookPair[ GREQUERYCHARPOSITIONS ].pDrv =
//                     (PFUN)(apfnDispatch[ NGreQueryCharPositions & 0x00FF ]);
//   GplEDCSetUp( &HookSet, NULL );  // Set up vectors
//   apfnDispatch[ NGreCharStringPos      & 0x00FF ] = (PFNL)GplCharStringPos;
//   apfnDispatch[ NGreCharString         & 0x00FF ] = (PFNL)GplCharString;
// }
//
//*****************************************************************************

BOOL APIENTRY GplEDCSetUp(
                  PGREDRVHOOKSET pHookTable,      // Ptr to hook table
                  PEDCGENDATA    pEDCGeneralData);// Ptr to general data


//*****************************************************************************
//
// FUNCTION: GplEDCRegisterGeneralData
//
// Called during EnableDC this function sets the offset in the instance data
// to where the pointer to EDCGENDATA is.
//
// If you do not do this and alter your engine hooking you will TRAP
//
// If pEDCGenData is null, it is assumed the caller used GplEDCSetUp with the
// pEDCGeneralData set to null and the hooktable is in EDC process space
//
//*****************************************************************************

BOOL APIENTRY GplEDCRegisterGeneralData(
                  PVOID  pInstance ,     // Ptr to instance data (pddc)
                  ULONG  ulOffset  ,     // Offset to GPLWDC data struct
                  PEDCGENDATA  pEDCGenData ); // Ptr to General data


//*****************************************************************************
//
// FUNCTION: GplCharStringPos
//
// The working routine.  The driver places this call instead of its own
// CharStringPos in the engine hook vector passed in during enable - fill
// physical device block.  When the engine calls this function it will scan
// string for any EDC ( provided by GplEDCSetGreChars ).  If none are found
// it will pass the call directly to the driver. The function gets the right
// driver entry point via GplEDCSetUp. If EDCs are found the function will:
//
// Query the driver for postions for EACH character via GREQUERYCHARPOSITIONS
// This means the driver  must know the widths for each character even EDCs
//
// The function then constructs a string of all the Non-EDCs with updated
// increment array.  This array will be adjusted so there are "holes" for
// the EDCs.  All attributes will be left with the  addition to CHS_VECTOR and
// CHS_LEAVEPOS.  The driver is called with the new parms.
//
// The function will now construct a string with only EDCs.  The increment
// array will be so the EDCs fall into the holes left by the driver.
// Now the engine is called to print this string. Options are again
// CHS_VECTOR and ~CHS_OPAQUE to turn off any background fills.
//
// Finally GreSetCurrentPosition is called to update current position for
// both engine and driver.
//
//*****************************************************************************

LONG APIENTRY GplCharStringPos ( );

//*****************************************************************************
//
// FUNCTION: GplCharString
//
// DESCRIPTION:
// Need to hook this as well since metafiles can turn charstringpos to a
// move and charstring.  There is no need to call back so
//
//*****************************************************************************

LONG APIENTRY GplCharString( );


//*****************************************************************************
//
// FUNCTION: GplEDCSetChars
//
// Called by driver at realize font set the list of EDCs
// Example in SetTextAttributes
//   if ( pddc->pEDCGenData->pEDCFontData == NULL )
//   {
//     LONG lGlyphs[1];
//     pddc->pEDCGenData->pEDCFontData = (PEDCFONTDATA)GplMemoryAlloc(
//           pddc->pdv->pDCHeap, DCFONTDATA_SIZE(1) );
//
//     lGlyphs[0] = EURO;
//     GplEDCSetChars( pddc,
//                     pddc->pEDCGenData->pEDCFontData,
//                     EDC_ACTIVE,
//                     0,
//                     1,
//                     lGlyphs);
//   }
//
//*****************************************************************************

BOOL APIENTRY GplEDCSetChars(
                  PVOID  pInstance ,     // Ptr to instance data (pddc)
                  PEDCFONTDATA  pEDCFontData,   // Ptr to Font data
                  ULONG  ulFlags   ,     // Flags, options
                  ULONG  ulHandle  ,     // Font handle so multiple fonts
                  LONG   lUGLCount ,     // Count of glyphs to handle
                  PLONG  plUGLList );    // list of glyphs

// Use the ACTION enum for GplEDCSetFlags(Action
typedef enum _ACTION { SET, CLEAR, QUERY } ACTION;

//*****************************************************************************
//
// FUNCTION: GplEDCSetFlags
//
// Helper function to set or clear flags in EDCData.  To use pass in ptr
// to instance data, optional handle, flag(s) and action - set or clear.
//
// If Action is Query then the RC will returb state of flag - TRUE = SET
// FALSE = CLEAR
//
// Example to make mapping active:
//    GplEDCSetFlags( pddc, 0, EDC_ACTIVE, SET );
// To turn off mapping:
//    GplEDCSetFlags( pddc, 0, EDC_ACTIVE, CLEAR );
//
//*****************************************************************************
BOOL APIENTRY GplEDCSetFlags(
                  PVOID   pInstance ,     // Ptr to instance data (pddc)
                  ULONG   ulHandle  ,     // Font handle so multiple fonts
                  ULONG   ulFlags   ,     // Flags
                  ACTION  Action    );    // Action - set or clear

//*****************************************************************************
//
// FUNCTION: GplEDCSetDCFlags
//
// Helper function to set or clear flags in EDC DC Data.  To use pass in ptr
// to instance data, optional handle, flag(s) and action - set or clear.
// Example to turn off font sub
//    GplEDCSetDCFlags( pddc, 0, EDC_DC_NOT_ACTIVE, SET );
// To font sub back on:
//    GplEDCSetDCFlags( pddc, 0, EDC_DC_NOT_ACTIVE, CLEAR );
// To QUERY font sub:
//    GplEDCSetDCFlags( pddc, 0, EDC_DC_NOT_ACTIVE, QUERY );
//
//*****************************************************************************

BOOL APIENTRY GplEDCSetDCFlags(
                  PVOID   pInstance ,     // Ptr to instance data (pddc)
                  ULONG   ulHandle  ,     // Font handle so multiple fonts
                  ULONG   ulFlags   ,     // Flags
                  ACTION  Action    );    // Action - set, clear, or query

//*****************************************************************************
//
// FUNCTION: GplEDCGetFontDataPtr
//
// Helper function used to get the font ptr from the general data
//
//*****************************************************************************

PEDCFONTDATA APIENTRY GplEDCGetFontDataPtr(
                  PVOID  pInstance );    // Ptr to instance data (pddc)

//*****************************************************************************
//
// FUNCTION: GplEDCSetFontDataPtr
//
// Helper function used to set the font ptr in the general data
// Use this function to switch font data between fonts or after it's freed
//
//*****************************************************************************

BOOL APIENTRY GplEDCSetFontDataPtr(
                  PVOID  pInstance ,     // Ptr to instance data (pddc)
                  PEDCFONTDATA  pEDCFontData ); // Ptr to Font data

//*****************************************************************************
//
// FUNCTION: GplGetCodePage
//
// Returns drivers current codepage
//
//*****************************************************************************

LONG APIENTRY GplGetCodePage( HDC   hdc       ,   // HDC
                              PVOID pInstance );  // Ptr to instance data
                  
//*****************************************************************************
//
// FUNCTION: GplEDCRemapGlyphs
//
// Called by driver at GreDeviceSetAttributes time this function will query
// engine for ABC Space for  glyphs
// typefield is: TEXT_ATTR | 0x00000010
//
//*****************************************************************************

BOOL APIENTRY GplEDCRemapGlyph(
                  HDC   hdc       ,   // HDC
                  PVOID pInstance );  // Ptr to instance data


//*****************************************************************************
//
// FUNCTION: GplCheckForEDC
//
// DESCRIPTION: Scans string for any EDCs, returns true if found, false if not
//              Scan will stop when zero byte or first EDC is encountered
//
//*****************************************************************************

BOOL APIENTRY GplCheckForEDC(
                  HDC   hdc        ,  // HDC
                  PVOID pInstance  ,  // ptr to instance data
                  LONG  lCharCount ,  // Count of characters
                  PCH   pchString  ); // Ptr to string


//*****************************************************************************
//
// FUNCTION: GplEDCGetABCSpace
//
// Given a glyph or character number will return ABC space in either font units
// or scaled units
//
// Example:
//   if ( sRemappedChar == EURO )
//   {
//     BOOL fRC;
//     ABCSPACE abc;
//     fRC = GplEDCGetABCSpace( pddc->hdc, pddc,
//                             EDC_SRC_UGL | EDC_NOTRANSFORM, EURO, &abc );
//     if ( fRC == TRUE )
//     {
//       CharWidth.glyph = EURO;
//       CharWidth.wx    = abc.lA + abc.ulB + abc.lC;
//       CharWidth.llx   = abc.lA;
//       CharWidth.urx   = abc.lA + abc.ulB;
//       pcdCurWidth = &CharWidth;
//     }
//   }
//
//*****************************************************************************

BOOL APIENTRY GplEDCGetABCSpace(
                  HDC        hdc       ,  // HDC
                  PVOID      pInstance ,  // Pddc
                  ULONG      ulOptions ,  //
                  LONG       lGlyph    ,  // Glyph num
                  PABCSPACE  pABCSpace ); // Ptr ABC space

// Flags for GplEDCGetABCSpace(ulOptions
#define    EDC_SRC_UGL       0x00000001
#define    EDC_SRC_CHAR      0x00000002
#define    EDC_TRANSFORM     0x00000004
#define    EDC_NOTRANSFORM   0x00000008


//=============================================================================
// How to use the Engine Drawn Characters (EDC) package:
//
// Basically the system works as follows:
//
// The driver first picks a "Backup" engine font which contains all the
// characters that are missing form the device font during RealizeFont.
//
// Next, during SetTextAttribute the driver decides based on the font and
// codepage what characters need to be drawn by the engine using the backup
// font.  These are known as EDC - Engine Drawn Characters
//
// When the engine calls the driver CharstringPos or CharString it will now
// call the GEL equivalent GplCharStringPos or GplCharString.  What these
// functions do is 1) Check for any EDC characters to draw in the string.  If
// there are none the EDC package will pass the call directly to the drivers.
// If there are the GplCharStringPos will query the driver for the character
// positions using QueryCharPositions.  Then the original string will be
// separated into two strings - one with device characters and one with EDCs.
// The device string will be sent to device driver with an altered increment
// vector to allow room for the EDCs.  Then the EDC string is sent to the
// engine also with an adjusted increment vector to place the EDC in the gaps
// left by the device fonts.  Finally the current position is set to where it
// should be at end of string.
//
//-----------------------------------------------------------------------------
//
// Implementation Notes
// --------------------
// First, during enable physical device time the printer driver must
// provide the EDC with 2 entry points - CharStringPos and
// QueryCharPositions.  This is done by filling in a GREDRVHOOKSET
// structure.  The struct is an array of GREDRVHOOKPAIR in each structure
// has an driver entry point an engine entry point.  For now the EDC just
// needs the 2 mentioned above.  If the driver does not change engine hooking
// depending on device then put the EDCGeneralData which contains hooktable
// in process space.  If engine hook table changes the put EDCGeneralData in
// the PDV so there is a copy for each DC.
// After calling GplEDCSetUp the driver hooks in
// both GplCharStringPos and GplCharString.  GplCharString just calls
// GplCharStringPos with a null starting position.  To the best of my knowlege
// all drivers work this way.
//
// After setting the EDC hook table via GplEDCSetUp you will need to call
// GplEDCRegisterGeneralData in Enable_DC to set the location of the
// EDCGeneralData ptr in the instance data which was not available to
// FillPhysical time.
//
// The next event is during RealizeFont time.  When a font is realized the
// driver must decide if and what engine font to pick. Once that is done
// The driver informs the engine of its selection by:
//
// Given the parms are:
//   GreRealizeFont(hdc, cmdCommand, pLogFont, pFont, pInstance, lFunction);
// Then
//   *((PSZ *)pFont) = (PSZ)(pLogFont->szFacename);
//
// would do it.  Normally when cmdCommand is RF_DEVICE_FONT pFont is a null
// ptr.  With the new engine, pFont points to NULL ptr.  The driver changes
// this pointed at pointer so it is a ptr to a string containing the name of
// the backup font.  At this time just using the same name works OK.  If the
// the engine does not know the font it will use the default font which has a
// Euro in it. It might be a good idea to flag the font as having a backup.
//
// Now the GplChar...  are hooked in, and a backup font(s) selected.  How does
// the driver tell GPLEDC package which glyphs its interested in?  When all the
// text attributes are set for a given series of Gpi calls are done the engine
// will call DeviceSetAttributes with bundle type of PRIM_CHAR_DONE (see above).
// At this time the engine has switched to requested font and its associated
// backup font.  Note - engine delays switching fonts when a CBB_SET is issued
// until driver signals OK with a good RC from set attribute.  Because of this
// delay any engine queries in SetTextAttributewill be against the old font.
// This was solved by PRIM_CHAR_DONE.  In PRIM_CHAR_DONE the driver should look
// at the font and current code page.  If it needs any EDC now is the time to
// specify them - given that the current font has a backup - that's     it is
// suggested to flag each font as it realized.
//
// GplEDCSetChars is used to indicate which GLYPHS are to be drawn by the
// engine.  Glyphs are from UGL that is they are absolute ids not dependant on
// code page mapping (see  32.3.2  Font Layout of "Presentation Manager
// Programming Reference Volume II - Messages and Related Information" for the
// actual list.  The lastest is count is around 505 - 383 is old!
// This Key - It is the drivers responsibility to allocate and manage the
// memory for holding GPLEDC data.  Use the EDCFONTDATA_SIZE(n) macro to get
// the size needed for n many glyphs.  One you have memory, place the ptr to in
// somewhere in the primary instance or pddc data.  One of the parameters for
// GplEDCSetChars is the offset from the start of instance data to the ptr.
// Setting the characters does not remap them.  You must set the EDC_NEED_REMAP
// or EDC_AUTOREMAP flag before use.  Or you can call GplEDCRemapGlyph directly.
//
// Options are:
//
// EDC_AUTOREMAP - remap characters when ever there is a possibility of needing
// a remap - inefficient but needed for poorly spaghetti drivers - use only
// really needed.
//
// EDC_SCALECHAR - if your driver does most of of its font work in scaled
// units this is good.  The Postscript and laserjet work in raw device units so
// they don't use it.
//
// EDC_ACTIVE - This means the EDC is initially active for the font.
//
// EDC_NEED_REMAP Remap the characters when first accessed.
//
// One can give a handle if you want several EDCData structs active at once.
// The way Postscript and Laserjet did it initially is just one EDCData and
// repmap each time the font or codepage is changed.  This works if you have
// the same glyph list for the system (eg just Euro).  You can use
// GplEDCSetFlags to turn on/off the EDC_ACTIVE flag depending on if EDC
// support is needed for a particular font/codepage combo.
//
// Once you have set up the glyph list you can have it repmapped each time the
// font or code page is changed.  Use GplEDCRemapGlyph to do this.  Don't
// forget to do this right after GplEDCSetChars as well unless is EDC_AUTOREMAP
// is on.
//
// Getting EDC size data
//
// Since the engine is drawing all the characters in the EDC list how does the
// driver know the size of each?  It has to call GplEDCGetABCSpace to do
// this. Essntially the driver calls the function with HDC, PDDC, flags, char,
// and ptr to an ABCSPACE struct.  The flags inicate if char is a glyph number
// or CharCode (mapped by a codepage).
// EDC_SRC_UGL - means char is glyph num.  EDC_SRC_CHAR is a mapped charcode.
// The flags also tell the function on what kinf of values to return.
// EDC_NOTRANSFORM means return raw device/font units.  The numbers come right
// from the AFM/OFM file.  They usally are based on coord system of 1000x1000
// and ment to be scaled.  EDC_TRANSFORM means give units scaled and
// transformed.  The size data is needed for QueryCharPositions, QueryTextBox,
// and QueryWidthTable.
//
//=============================================================================

#endif
