/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = JOBFEAT.H
 *
 * DESCRIPTION = Public output memory header for genplib.lib
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 02/18/97
 * AUTHOR  = Larry Moore
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_jobfeat_h
#define _gpl_jobfeat_h

#pragma pack( 1 )





/*****************************************************************************\
** DEFINES                                                                   **
\*****************************************************************************/
#define CONTEXTHELP_TEXTLIMIT    128

#define GJM_FORMMODIFIED         1

#define GJMF_PAGESIZE            0
#define GJMF_IMGAREA             1
#define GJMF_ORIENTATION         2

#define GJT_PORTRAIT             0
#define GJT_LANDSCAPE            1

#define GJNI_QUERY_DLG_HANDLE          0x0001
#define GJNI_QUERY_NOTEBOOK_HANDLE     0x0002
#define GJNI_QUERY_CSH_HANDLE          0x0003
#define GJNI_QUERY_MODULE_HANDLE       0x0004
#define GJNI_QUERY_MEMORY_HEAP         0x0005
#define GJNI_QUERY_PAGE_SIZE           0x0006
#define GJNI_QUERY_IMG_AREA            0x0007
#define GJNI_QUERY_ORIENTATION         0x0008
#define GJNI_QUERY_HELP_HWND           0x0009
#define GJNI_QUERY_DEVICE_NAME         0x000A

#define GJNI_NUM_OF_QUERIES            9
#define GJNI_FIRST_QUERY               GJNI_QUERY_DLG_HANDLE

#define GJPI_QUERY_PAGE_HANDLE         0x1001
#define GJPI_QUERY_CONTROL_BLOCK       0x1002
#define GJPI_QUERY_DIALOG_BUFFER       0x1003
#define GJPI_QUERY_NOTEBOOK_INSTANCE   0x1004
#define GJPI_QUERY_HELP_PANEL_ID       0x1005
#define GJPI_QUERY_CALLBACK_PROC       0x1006
#define GJPI_QUERY_CALLBACKPROC_PARAM  0x1007

#define GJPI_NUM_OF_QUERIES            5
#define GJPI_FIRST_QUERY               GJPI_QUERY_PAGE_HANDLE

#define GJF_NUP_ROTATE             1
#define GJF_NUP_NO_ROTATE          0

#define GJF_SHOW_PAGE              1
#define GJF_HIDE_PAGE              0





/*****************************************************************************\
** TYPEDEFS                                                                  **
\*****************************************************************************/
/*
** HGJOI - Handle to a GENPLIB Job Features Output Instance.
*/
typedef ULONG             HGJOI;
typedef HGJOI           *PHGJOI;
// Handle - GENPLIB Job Features Notebook Instance.
typedef ULONG             HGJNI;
typedef HGJNI           *PHGJNI;

// Handle - GENPLIB Job Features Page Instance.
typedef ULONG             HGJPI;
typedef HGJPI           *PHGJPI;


/*
** GENPLIB Job Features context header.
** This header must be the first element in any GENPLIB Job Features structure.
** This structure allows for buffer expansion and contains information which
** identifies a unique structure and if that structure has been initialized.
** NOTE: For ulVersionTotal, this is provided by the user.  This cannot be 0.
** The version number does not have to be modifed as long as new data is added
** to the 'bottom' of the buffer.  This field is provided in case the structure
** was modified in a way that a simple copy will not work.
*/
#define GJF_NAME_SIZE               16
typedef struct _GJFC
{
  CHAR   aAppName[ GJF_NAME_SIZE ];    // Application name
  CHAR   aKeyName[ GJF_NAME_SIZE ];    // Key name

  ULONG  cbTotal;                // Total size of data including this struct.
  ULONG  cbThis;                 // Size of this structure only.

  ULONG  ulVersionTotal;         // Version of entire buffer
  ULONG  ulFlags;                // Internal flags
         #define GJF_OUTPUT_INSTANCE_ACTIVE   1
         #define GJF_DIALOG_INSTANCE_ACTIVE   2

  ULONG ulReserved;              // Reserved - must be 0
} GJFC, *PGJFC, **PPGJFC;
// Sets, resets, and tests the 'ulFlags' field in GJFC.
#define GJFC_SET_FLAG( f, b )         (((PGJFC) (f))->ulFlags |= (b))
#define GJFC_RESET_FLAG( f, b )       (((PGJFC) (f))->ulFlags &= ~(b))
#define GJFC_FLAG_IS_SET( f, b )      (((PGJFC) (f))->ulFlags & (b))

typedef struct _SPD
{
  HWND   hPage;                // Sample page handle
  HPS    hPS;                  // Handle to the sample page presentation space
  RECTL  rclWindowSize;        // Size of the physical sample page frame
  POINTL ptlMidpoint;          // Mid point within the sample page
  RECTL  rclPageSize;          // Size of the virtual page within the window
  HRGN   hRgn;                 // Handle to the page limit clipping region
  PFNWP  pOldDlgProc;          // Old dialog procedure pointer
} SPD, *PSPD;

// GENPLIB Job Features Callback Procedure
typedef MRESULT (APIENTRY GJCP) (PGJFC, PVOID);
typedef GJCP    *PGJCP, **PPGJCP;





/*****************************************************************************\
** FUNCTION PROTOTYPES                                                       **
\*****************************************************************************/
LONG   APIENTRY GplJFInitCB( PVOID, USHORT, PSZ, ULONG );
LONG   APIENTRY GplJFCopyCB( PVOID, PVOID, PSZ );
HGJOI  APIENTRY GplJFCreateInstance( PVOID, ULONG, PVOID, PSZ );
LONG   APIENTRY GplJFDeleteInstance( HGJOI, PSZ );
LONG   APIENTRY GplJFVerifyCB( PGJFC, PSZ );
HGJNI  APIENTRY GplJFCreateNotebookInstance( HWND, ULONG, HWND, HWND, HMODULE,
                                             PSZ, PVOID );
LONG   APIENTRY GplJFQueryNBInstParamList( HGJNI, PULONG, ULONG, ULONG );
LONG   APIENTRY GplJFQueryPageInstParamList( HGJPI, PULONG, ULONG, ULONG );
LONG   APIENTRY GplJFDestroyNotebookInstance( HGJNI );
HGJPI  APIENTRY GplJFCreatePageInstance( PVOID, HGJNI, ULONG, ULONG, PGJCP,
                                         PVOID, PPVOID );
HWND   APIENTRY GplJFDisplayPage( HGJPI, PGDPTMPL );
LONG   APIENTRY GplJFDestroyPageInstance( HWND, HGJPI );
HWND   APIENTRY GplJFSetContextHelp( HGJPI, HWND, ULONG );
ULONG  APIENTRY GplJFShowContextHelp( HGJPI, MPARAM );
LONG   APIENTRY GplJFResetControlBlock( PVOID, BOOL );
LONG   APIENTRY GplJFUpdateFormSettings( HGJNI, PMPARAM, ULONG, ULONG );
HGJPI  APIENTRY GplJFSetPageParams( HWND, MPARAM );
HGJPI  APIENTRY GplJFQueryPageParams( HWND, PPVOID );
HGJPI  APIENTRY GplJFQueryPageCB( HWND, PPVOID );
LONG   APIENTRY GplJFQueryCBSize( PVOID );
BOOL   APIENTRY GplJFWriteProfile( PVOID, PSZ, PSZ );
BOOL   APIENTRY GplJFReadProfile( PVOID, PSZ, PSZ );
HWND   APIENTRY GplJFInitSamplePage( HWND, PSPD, ULONG, PFNWP, MPARAM );
PSIZEL APIENTRY GplJFDrawSamplePage( HGJPI, PSPD );
BOOL   APIENTRY GplJFDestroySamplePage( PSPD );
BOOL   APIENTRY GplJFDisplayHelpPanel( HWND, ULONG );
ULONG  APIENTRY GplJFIsPageJobFeature( HWND, ULONG );
LONG   APIENTRY GplJFQueryCB( PVOID, LONG, PSZ, PPGJFC );
PGJCP  APIENTRY GplJFQueryCallbackProc( HGJPI, PPGJCP, PPVOID );
LONG   APIENTRY GplJFShowPage( HGJPI, LONG );

#endif
