/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = MEMINI.H
 *
 * DESCRIPTION = Functions dealing with the memory ini file format.
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 09/01/97
 * AUTHOR  = Mark Hamzy
 *
 * FUNCTIONS
 *
 *
 * NOTES
 *
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_memini_h
#define _gpl_memini_h

/*************** I M P O R T A N T pragma follows ! ! ! ***************/
#pragma pack(1)

typedef ULONG   MIHANDLE;
typedef PULONG PMIHANDLE;

/* This version is composed of an array of MEMINI structures with
** a NUL MEMINI terminator.
*/
#define MEMINI_VERSION         1

/* NOTE ABOUT FUTURE VERSIONS:
**
**    You may or may not want to keep the array of MEMINI structures at
**    the beginning for pseudo-backward compatibility.  You can then add
**    some indexing information for faster searching methods after the
**    list.
*/

/* This is an eight bit value stored in the following format
**     87654321
**     xxxx     - flags
**         xxxx - type (1 - 15)
*/
typedef unsigned char   EMEMINITYPE;
typedef unsigned char *PEMEMINITYPE;

#define MEMINITYPE_FLAG_MASK   0xF0
#define EMEMINIFLAG_PRIVATE    0x80           // Private... not to be transfered (app is responsible!)

#define MEMINITYPE_TYPE_MASK   0x0F
#define EMEMINITYPE_ULONG      0x01           // Unsigned 32-bit number
#define EMEMINITYPE_LONG       0x02           // Signed 32-bit number
#define EMEMINITYPE_STRING     0x03           // String
#define EMEMINITYPE_BINDATA    0x04           // Binary data

typedef struct _MemIni {
   LONG    lOffNext;                          // offset to next
   LONG    cbData;                            // size of application data
   BYTE    eDataType;                         // type of application data
   CHAR    achApp[2];                         // Nul terminated Application name
                                              // Nul terminated Key         name
                                              // Binary application data
} MEMINI, *PMEMINI;

#define MEMINI_SIG     0x54626F4A             // 'JobT'
struct _ExtraJobProperties {
   LONG    cb;                                // size
   ULONG   ulSig;                             // signature
   ULONG   ulVersion;                         // version
   MEMINI  StartIni;                          // Start of ini file
};
typedef struct _ExtraJobProperties EXJOBPROPERTIES, *PEXJOBPROPERTIES;

/*************** I M P O R T A N T pragma follows ! ! ! ! ***************/
#pragma pack()

#define NEXT_MEMINI(p) ((PMEMINI)((PBYTE)p+p->lOffNext))

/* Defines for GplMemIniCreateInstance ()
*/
#define MEMINI_CI_READ_ONLY         0x00000001

/* Defines for GplMemIniDeleteInstance ()
*/
#define MEMINI_OP_KEEP              1
#define MEMINI_OP_DISCARD           2
#define MEMINI_OP_KEEP_INTERMEDIATE 3

//  App: NOTIFY          Key: PLACE           Data: Place Name
//       NOTIFY               EMAIL                 e-mail id

#define MEMINI_NOTIFY_PLACE_APPNAME   "NOTIFY"
#define MEMINI_NOTIFY_PLACE_KEYNAME   "PLACE"
#define MEMINI_NOTIFY_EMAIL_APPNAME   "NOTIFY"
#define MEMINI_NOTIFY_EMAIL_KEYNAME   "EMAIL"

/* Function prototypes...
*/
PEXJOBPROPERTIES APIENTRY GplMemIniCompliant        (PDRIVDATA    pDD,
                                                     ULONG        cbMaxSizeBuffer);
APIRET           APIENTRY GplMemIniCreateInstance   (PDRIVDATA    pDDIn,
                                                     PMIHANDLE    pMIHandle,
                                                     ULONG        cbSizeBuffer,
                                                     ULONG        ulOptions);
ULONG            APIENTRY GplMemIniDeleteInstance   (MIHANDLE     MIHandle,
                                                     ULONG        ulOp,
                                                                  ...);
ULONG            APIENTRY GplMemIniCurrentSize      (MIHANDLE     MIHandle);
BOOL             APIENTRY GplMemIniAddAppKeyData    (MIHANDLE     MIHandle,
                                                     PSZ          pszAppName,
                                                     PSZ          pszKeyName,
                                                     PBYTE        pbData,
                                                     LONG         cbData,
                                                     EMEMINITYPE  eType,
                                                     ULONG        cbSizeBuffer);
BOOL             APIENTRY GplMemIniRemoveAppKeyData (MIHANDLE     MIHandle,
                                                     PSZ          pszAppName,
                                                     PSZ          pszKeyName);
PBYTE            APIENTRY GplMemIniFindAppKeyData   (MIHANDLE     MIHandle,
                                                     PSZ          pszAppName,
                                                     PSZ          pszKeyName,
                                                     PLONG        pcbData,
                                                     PEMEMINITYPE peType);

#endif
