/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1992 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 source code is provided to you solely for       */
/*    the purpose of assisting you in your development of OS/2 device        */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Developer Connection Device Driver       */
/*    Source Kit for OS/2. This Copyright statement may not be removed.      */
/*                                                                           */
/*****************************************************************************/
/**************************************************************************
 *
 * SOURCE FILE NAME = SAMPAGE.H
 *
 * DESCRIPTION = Public output Sample Page header for GENPLIB.LIB
 *
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation. This sample code is not
 * part of any standard or IBM product and is provided to you solely
 * for  the purpose of assisting you in the development of your
 * presentation drivers.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have been
 * advised of the possibility of such damages.
 *
 * VERSION = V1.0
 * DATE    = 04/04/96
 * AUTHOR  = Larry Moore
 *
 * FUNCTIONS
 *
 *
 * NOTES
 * A Sample page is a rectangular control that is used to display a sample
 * output on how a page might look with the control's features implemented.
 * For example, the sample page for Watermarks displays a watarmark on the
 * page.  For N-UP, the sample page displays how the N-UP pages would look
 * like on a sheet of paper.
 * The sample page consists of a rectangular frame, painted white, looking like
 * a sheet of paper.  On this 'paper' are lines drawn to look like lines of
 * text viewed from a distance.  Sample pages even support 'indenting', which
 * simply consists of incrementing the starting X-coordinate by a certain
 * value before drawing the line.
 *
 * STRUCTURES
 *
 *
 * EXTERNAL REFERENCES
 *
 *
 * EXTERNAL FUNCTIONS
 *
 *
*/

#ifndef _gpl_sampage_h
#define _gpl_sampage_h

/*****************************************************************************\
** DEFINES                                                                   **
\*****************************************************************************/
/*
** This macro returns a pointer to the GSPH structure.  You will notice that
** all dialog headers that use the sample page code must define a GSPH
** structure as the second field.  This macro is the reason     - the pointer
** starts off at the beginning of the structure and is incremented the size
** of the page information structure.
*/
#define QUERY_PGSPH( p )   (PGSPH) ((PBYTE) (p) + sizeof( GPINFO ))

#define SPO_PORTRAIT    0
#define SPO_LANDSCAPE   1





/*****************************************************************************\
** TYPEDEFS                                                                  **
\*****************************************************************************/
/*
** This macros defines the user-defined function.  Although the Sample Page
** code draws the text lines, the user may want to make additional drawings
** to the page.  This user-defined function is called by the Sample Page
** code to allow the user to draw to the sample page.  The sample page
** function is defined in GplSmpgInit().
*/
typedef LONG ( APIENTRY GSP_UF )( HWND, PVOID );
typedef GSP_UF  *PGSP_UF;



/*
** GENPLIB Sample Page Header
** This structure makes up the Sample Page Header.  This structure contains
** a minimum number of fields needed to maintain the Sample Page.  The user
** may create another structure that contains additional Sample Page
** Information (see GDWSPI in GPLWM.H for example).
*/
typedef struct _GSPH
{
  HWND  hSample;                 // Sample page dialog
  HPS   hPS;                     // Page pres. space handle
  RECTL rclPage;                 // Rectangle size of the page

  LONG  lParaIndentSize;         // Size of indentation of lines for para's
  LONG  lSpaceBetweenLines;      // Space size between lines
  LONG  lLRMarginSize;           // Left and right margin size
  LONG  lTBMarginSize;           // Top and bottom margin size
  LONG  lNumOfParaLines;         // # of lines per paragraph

  PGSP_UF pInitFn;               // Initialization function - user defined
  PGSP_UF pPaintFn;              // Paint function - user defined
} GSPH, *PGSPH;





/*****************************************************************************\
** FUNCTION PROTOTYPES                                                       **
\*****************************************************************************/
extern VOID APIENTRY GplSmpgInit( HWND, ULONG, PVOID, LONG, LONG, LONG, LONG,
                                  LONG, PGSP_UF, PGSP_UF );
extern VOID APIENTRY GplSmpgDrawLines( PGSPH, LONG );

#endif
